/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.backgroundjob.ws;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.backgroundjob.health.HealthIndicator;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@RunWith(MockitoJUnitRunner.class)
public class AbstractHealthServletTest {
    @Spy
    private List<HealthIndicator> healthIndicators = new ArrayList<>();

    @InjectMocks
    private AbstractHealthServlet subject;

    @Test
    public void testEmpty() throws IOException {

        // Given
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        // When
        subject.doGet(request, response);

        // Then
        Assert.assertEquals(200, response.getStatus());
        Assert.assertEquals("{}", response.getContentAsString());
    }

    @Test
    public void testWithHealthIndication() throws IOException {
        // Given
        HealthIndicator mockIndication1 = Mockito.mock(HealthIndicator.class);
        Mockito.when(mockIndication1.getHealthInfo()).thenReturn(new HealthIndication(500, "\"test1\": \"abc\""));
        HealthIndicator mockIndication2 = Mockito.mock(HealthIndicator.class);
        Mockito.when(mockIndication2.getHealthInfo()).thenReturn(new HealthIndication(200, "\"test2\": \"xyz\""));
        healthIndicators.add(mockIndication1);
        healthIndicators.add(mockIndication2);
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        // When
        subject.doGet(request, response);

        // Then
        Assert.assertEquals(500, response.getStatus());
        Assert.assertEquals("{\"test1\": \"abc\",\"test2\": \"xyz\"}", response.getContentAsString());

    }
}
