/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.backgroundjob.ws;

import dk.nsp.backgroundjob.service.StackedOperationsService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import static org.junit.Assert.assertEquals;

public class AbstractOperationsStatusServletTest {

    private StackedOperationsService operationsService;
    private AbstractOperationsStatusServlet abstractOperationsStatusServlet;

    @Before
    public void setUp() {
        operationsService = Mockito.mock(StackedOperationsService.class);
        abstractOperationsStatusServlet = new AbstractOperationsStatusServlet() {
            @Override
            protected StackedOperationsService getOperationsService() {
                return operationsService;
            }
        };
    }

    @Test
    public void doGet_WhenLatestRunSucceeded_Returns200() {
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();
        Mockito.when(operationsService.latestRunSuccessful()).thenReturn(true);

        abstractOperationsStatusServlet.doGet(request, response);

        assertEquals(200, response.getStatus());
    }

    @Test
    public void doGet_WhenLatestRunFailed_Returns500() {
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();
        Mockito.when(operationsService.latestRunSuccessful()).thenReturn(false);

        abstractOperationsStatusServlet.doGet(request, response);

        assertEquals(500, response.getStatus());
    }
}