/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.auditlog;

import dk.sds.nsp.audit.AuditBuilder;
import dk.sds.nsp.audit.AuditBuilder.TypeOfInformation;
import dk.sundhedsdatastyrelsen.livstestamente.domain.LivingWillVO;
import dk.sundhedsdatastyrelsen.livstestamente.domain.TreatmentWillVO;
import dk.sundhedsdatastyrelsen.livstestamente.context.ServiceActor;

import java.util.function.Supplier;

public class AuditLogger {
    private static final String COMPONENT = "BTR";

    private static final String GET_TREATMENT_WILL_CONTEXT = "getTreatmentWill";
    private static final String GET_TREATMENT_WILL_WITH_ONLY_FORCED_TREATMENT_CONTEXT = "getTreatmentWillWithOnlyForcedTreatment";
    private static final String HAS_TREATMENT_WILL_CONTEXT = "hasTreatmentWill";
    private static final String CREATE_TREATMENT_WILL_CONTEXT = "createTreatmentWill";
    private static final String UPDATE_TREATMENT_WILL_CONTEXT = "updateTreatmentWill";
    private static final String DELETE_TREATMENT_WILL_CONTEXT = "deleteTreatmentWill";
    private static final String UPGRADE_TO_TREATMENT_WILL_CONTEXT = "upgradeToTreatmentWill";
    private static final String UPDATE_LIVING_WILL_CONTEXT = "updateLivingWill";
    private static final String DELETE_LIVING_WILL_CONTEXT = "deleteLivingWill";
    private static final String GET_LIVING_WILL_CONTEXT = "getLivingWill";
    private static final String HAS_LIVING_WILL_CONTEXT = "hasLivingWill";
    private static final String VALIDATE_CPR_CONTEXT = "validateCpr";
    private final Supplier<AuditBuilder> auditBuilderSupplier;

    public AuditLogger(Supplier<AuditBuilder> auditBuilderSupplier) {
        this.auditBuilderSupplier = auditBuilderSupplier;
    }

    public void logGetTreatmentWill(TreatmentWillVO treatmentWill, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        logTreatmentWill(auditBuilder, GET_TREATMENT_WILL_CONTEXT, treatmentWill);
        logServiceActor(auditBuilder, GET_TREATMENT_WILL_CONTEXT, serviceActor);
    }

    public void logGetTreatmentWillWithOnlyForcedTreatment(TreatmentWillVO treatmentWill, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        logTreatmentWill(auditBuilder, GET_TREATMENT_WILL_WITH_ONLY_FORCED_TREATMENT_CONTEXT, treatmentWill);
        logServiceActor(auditBuilder, GET_TREATMENT_WILL_WITH_ONLY_FORCED_TREATMENT_CONTEXT, serviceActor);
    }

    public void logHasTreatmentWill(String personIdentifier, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        log(auditBuilder, HAS_TREATMENT_WILL_CONTEXT, Type.CPR, personIdentifier);
        logServiceActor(auditBuilder, HAS_TREATMENT_WILL_CONTEXT, serviceActor);
    }

    public void logCreateTreatmentWill(TreatmentWillVO treatmentWill, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        logTreatmentWill(auditBuilder, CREATE_TREATMENT_WILL_CONTEXT, treatmentWill);
        logServiceActor(auditBuilder, CREATE_TREATMENT_WILL_CONTEXT, serviceActor);
    }

    public void logUpdateTreatmentWill(TreatmentWillVO treatmentWill, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        logTreatmentWill(auditBuilder, UPDATE_TREATMENT_WILL_CONTEXT, treatmentWill);
        logServiceActor(auditBuilder, UPDATE_TREATMENT_WILL_CONTEXT, serviceActor);
    }

    public void logDeleteTreatmentWill(String personIdentifier, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        log(auditBuilder, DELETE_TREATMENT_WILL_CONTEXT, Type.CPR, personIdentifier);
        logServiceActor(auditBuilder, DELETE_TREATMENT_WILL_CONTEXT, serviceActor);
    }

    public void logUpgradeToTreatmentWill(TreatmentWillVO treatmentWill, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        logTreatmentWill(auditBuilder, UPGRADE_TO_TREATMENT_WILL_CONTEXT, treatmentWill);
        logServiceActor(auditBuilder, UPGRADE_TO_TREATMENT_WILL_CONTEXT, serviceActor);
    }

    public void logUpdateLivingWill(LivingWillVO livingWill, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        logLivingWill(auditBuilder, UPDATE_LIVING_WILL_CONTEXT, livingWill);
        logServiceActor(auditBuilder, UPDATE_LIVING_WILL_CONTEXT, serviceActor);
    }

    public void logDeleteLivingWill(String personIdentifier, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        log(auditBuilder, DELETE_LIVING_WILL_CONTEXT, Type.CPR, personIdentifier);
        logServiceActor(auditBuilder, DELETE_LIVING_WILL_CONTEXT, serviceActor);
    }

    public void logGetLivingWill(LivingWillVO livingWill, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        logLivingWill(auditBuilder, GET_LIVING_WILL_CONTEXT, livingWill);
        logServiceActor(auditBuilder, GET_LIVING_WILL_CONTEXT, serviceActor);
    }

    public void logHasLivingWill(String personIdentifier, ServiceActor serviceActor) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        log(auditBuilder, HAS_LIVING_WILL_CONTEXT, Type.CPR, personIdentifier);
        logServiceActor(auditBuilder, HAS_LIVING_WILL_CONTEXT, serviceActor);
    }

    public void logCprDoesNotExist(String cpr) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        log(auditBuilder, VALIDATE_CPR_CONTEXT, Type.CPR_DOES_NOT_EXIST, cpr);
    }

    public void logCprInactive(String cpr) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        log(auditBuilder, VALIDATE_CPR_CONTEXT, Type.CPR_INACTIVE, cpr);
    }

    private void logServiceActor(AuditBuilder auditBuilder, String context, ServiceActor serviceActor) {
        log(auditBuilder, context, Type.ACTOR_CPR, serviceActor.getPersonIdentifier());
        log(auditBuilder, context, Type.ACTOR_FIRSTNAME, serviceActor.getFirstName());
        log(auditBuilder, context, Type.ACTOR_LASTNAME, serviceActor.getLastName());
        log(auditBuilder, context, Type.ACTOR_USERTYPE, serviceActor.getUserType());
        log(auditBuilder, context, Type.ACTOR_AUTHORISATION_NUMBER, serviceActor.getAuthorisationNumber());
        log(auditBuilder, context, Type.ACTOR_EDUCATION_CODE, serviceActor.getEducationCode());
        log(auditBuilder, context, Type.ACTOR_ORGANISATION_ID, serviceActor.getOrganizationId());
        log(auditBuilder, context, Type.ACTOR_ORGANISATION_ID_SOURCE, serviceActor.getOrganizationIdSource());
        log(auditBuilder, context, Type.ACTOR_ORGANISATION_NAME, serviceActor.getOrganizationName());
    }

    private void logTreatmentWill(AuditBuilder auditBuilder, String context, TreatmentWillVO treatmentWill) {
        log(auditBuilder, context, Type.PID, treatmentWill.getTreatmentWillPID());
        log(auditBuilder, context, Type.CPR, treatmentWill.getPersonIdentifier());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_DYING, treatmentWill.isNoLifeProlongingIfDying());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_DYING_ACCEPTANCE_NEEDED, treatmentWill.getNoLifeProlongingIfDyingAcceptanceNeeded());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED, treatmentWill.isNoLifeProlongingIfSeverelyDegraded());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED_ACCEPTANCE_NEEDED, treatmentWill.getNoLifeProlongingIfSeverelyDegradedAcceptanceNeeded());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_SEVERE_PAIN, treatmentWill.isNoLifeProlongingIfSeverePain());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_SEVERE_PAIN_ACCEPTANCE_NEEDED, treatmentWill.getNoLifeProlongingIfSeverePainAcceptanceNeeded());
        log(auditBuilder, context, Type.NO_FORCED_TREATMENT_IF_INCAPABLE, treatmentWill.isNoForcedTreatmentIfIncapable());
        log(auditBuilder, context, Type.NO_FORCED_TREATMENT_IF_INCAPABLE_ACCEPTANCE_NEEDED, treatmentWill.getNoForcedTreatmentIfIncapableAcceptanceNeeded());
    }

    private void logLivingWill(AuditBuilder auditBuilder, String context, LivingWillVO livingWill) {
        log(auditBuilder, context, Type.PID, livingWill.getLivingWillPID());
        log(auditBuilder, context, Type.CPR, livingWill.getPersonIdentifier());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_DYING, livingWill.isNoLifeProlongingIfDying());
        log(auditBuilder, context, Type.NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED, livingWill.isNoLifeProlongingIfSeverelyDegraded());
    }

    private void log(AuditBuilder auditBuilder, String context, Type type, Object value) {
        if (value != null) {
            auditBuilder.addAuditInformation(COMPONENT, context, type.typeOfInformation, type.value, value);
        }
    }

    private enum Type {
        PID("pid", TypeOfInformation.NonPersonalInformation),
        CPR("cpr", TypeOfInformation.RegularPersonalInformation),
        NO_LIFE_PROLONGING_IF_DYING("no-life-prolonging-if-dying", TypeOfInformation.SensitivePersonalInformation),
        NO_LIFE_PROLONGING_IF_DYING_ACCEPTANCE_NEEDED("no-life-prolonging-if-dying-acceptance-needed", TypeOfInformation.SensitivePersonalInformation),
        NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED("no-life-prolonging-if-severely-degraded", TypeOfInformation.SensitivePersonalInformation),
        NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED_ACCEPTANCE_NEEDED("no-life-prolonging-if-severely-degraded-acceptance-needed", TypeOfInformation.SensitivePersonalInformation),
        NO_LIFE_PROLONGING_IF_SEVERE_PAIN("no-life-prolonging-if-severe-pain", TypeOfInformation.SensitivePersonalInformation),
        NO_LIFE_PROLONGING_IF_SEVERE_PAIN_ACCEPTANCE_NEEDED("no-life-prolonging-if-severe-pain-acceptance-needed", TypeOfInformation.SensitivePersonalInformation),
        NO_FORCED_TREATMENT_IF_INCAPABLE("no-forced-treatment-if-incapable", TypeOfInformation.SensitivePersonalInformation),
        NO_FORCED_TREATMENT_IF_INCAPABLE_ACCEPTANCE_NEEDED("no-forced-treatment-if-incapable-acceptance-needed", TypeOfInformation.SensitivePersonalInformation),
        ACTOR_CPR("actor-cpr", TypeOfInformation.RegularPersonalInformation),
        ACTOR_FIRSTNAME("actor-firstname", TypeOfInformation.RegularPersonalInformation),
        ACTOR_LASTNAME("actor-lastname", TypeOfInformation.RegularPersonalInformation),
        ACTOR_USERTYPE("actor-usertype", TypeOfInformation.RegularPersonalInformation),
        ACTOR_AUTHORISATION_NUMBER("actor-authorisation-number", TypeOfInformation.RegularPersonalInformation),
        ACTOR_EDUCATION_CODE("actor-education-code", TypeOfInformation.RegularPersonalInformation),
        ACTOR_ORGANISATION_ID("actor-organisation-id", TypeOfInformation.RegularPersonalInformation),
        ACTOR_ORGANISATION_ID_SOURCE("actor-organisation-id-source", TypeOfInformation.RegularPersonalInformation),
        ACTOR_ORGANISATION_NAME("actor-organisation-name", TypeOfInformation.RegularPersonalInformation),
        CPR_DOES_NOT_EXIST("cpr-does-not-exist", TypeOfInformation.RegularPersonalInformation),
        CPR_INACTIVE("cpr-inactive", TypeOfInformation.RegularPersonalInformation);

        private final String value;
        private final TypeOfInformation typeOfInformation;

        Type(String value, TypeOfInformation typeOfInformation) {
            this.value = value;
            this.typeOfInformation = typeOfInformation;
        }
    }
}
