/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.auditlog;

import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

public class LogEntry {
    private Logger logger;
    private LogLevel logLevel;
    private List<String> parameters;
    private Throwable throwable;

    LogEntry(Logger logger, LogLevel logLevel, String message) {
        this(logger, logLevel, message, null);
    }

    LogEntry(Logger logger, LogLevel logLevel, String message, Throwable throwable) {
        this.logger = logger;
        this.logLevel = logLevel;
        this.throwable = throwable;

        parameters = new ArrayList<>();
        String errorMessage = throwable != null ? throwable.getMessage() : null;
        if (errorMessage != null && !errorMessage.isEmpty()) {
            addParameter("error", throwable.getMessage());
        }
        if (message != null && !message.isEmpty() && !message.equals(errorMessage)) {
            addParameter("msg", message);
        }
    }

    public void addParameter(String name, String value) {
        parameters.add(name + "=" + value);
    }

    private String getLogEntry() {
        return String.join(", " ,parameters);
    }

    public void commit() {
        if (throwable == null) {
            if (logLevel == LogLevel.INFO) {
                logger.info(getLogEntry());
            } else if (logLevel == LogLevel.WARN) {
                logger.warn(getLogEntry());
            } else if (logLevel == LogLevel.DEBUG) {
                logger.debug(getLogEntry());
            } else if (logLevel == LogLevel.ERROR) {
                logger.error(getLogEntry());
            }
        } else {
            if (logLevel == LogLevel.INFO) {
                logger.info(getLogEntry(), throwable);
            } else if (logLevel == LogLevel.WARN) {
                logger.warn(getLogEntry(), throwable);
            } else if (logLevel == LogLevel.DEBUG) {
                logger.debug(getLogEntry(), throwable);
            } else if (logLevel == LogLevel.ERROR) {
                logger.error(getLogEntry(), throwable);
            }
        }
    }
}
