/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.context;

import java.util.Objects;

import dk.sds.nsp.security.SecurityContext.Organisation.OrganisationIdentifierFormat;
import dk.sundhedsdatastyrelsen.livstestamente.domain.UserType;

/**
 * Data Access Object for ServiceActor table
 */

public class ServiceActor {
    private UserType userType;

    private Long serviceActorPID;
    private String personIdentifier;
    private String firstName;
    private String lastName;
    private String authorisationNumber;
    private String educationCode;
    private String specialityCode;
    private String specialityCodeSource;
    private String organizationId;
    private OrganisationIdentifierFormat organizationIdSource;
    private String organizationName;
    private String userTitle;

    public ServiceActor(UserType userType) {
        this.userType = userType;
    }
    
    public UserType getUserType() {
		return userType;
	}

	public String getName() {
        return getFirstName() + " " + getLastName();
    }

    public Long getServiceActorPID() {
        return serviceActorPID;
    }

    public void setServiceActorPID(Long serviceActorPID) {
        this.serviceActorPID = serviceActorPID;
    }

    public String getPersonIdentifier() {
        return personIdentifier;
    }

    public void setPersonIdentifier(String personIdentifier) {
        this.personIdentifier = personIdentifier;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getAuthorisationNumber() {
        return authorisationNumber;
    }

    public void setAuthorisationNumber(String authorisationNumber) {
        this.authorisationNumber = authorisationNumber;
    }

    public String getEducationCode() {
        return educationCode;
    }

    public void setEducationCode(String educationCode) {
        this.educationCode = educationCode;
    }

    public String getSpecialityCode() {
        return specialityCode;
    }

    public void setSpecialityCode(String specialityCode) {
        this.specialityCode = specialityCode;
    }

    public String getSpecialityCodeSource() {
        return specialityCodeSource;
    }

    public void setSpecialityCodeSource(String specialityCodeSource) {
        this.specialityCodeSource = specialityCodeSource;
    }

    public String getOrganizationId() {
        return organizationId;
    }

    public void setOrganizationId(String organizationId) {
        this.organizationId = organizationId;
    }

    public OrganisationIdentifierFormat getOrganizationIdSource() {
        return organizationIdSource;
    }

    public void setOrganizationIdSource(OrganisationIdentifierFormat organizationIdSource) {
        this.organizationIdSource = organizationIdSource;
    }

    public String getOrganizationName() {
        return organizationName;
    }

    public void setOrganizationName(String organizationName) {
        this.organizationName = organizationName;
    }

    public String getUserTitle() {
        return userTitle;
    }

    public void setUserTitle(String userTitle) {
        this.userTitle = userTitle;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        ServiceActor that = (ServiceActor) o;
        return Objects.equals(serviceActorPID, that.serviceActorPID) &&
                Objects.equals(personIdentifier, that.personIdentifier) &&
                Objects.equals(firstName, that.firstName) &&
                Objects.equals(lastName, that.lastName) &&
                Objects.equals(authorisationNumber, that.authorisationNumber) &&
                Objects.equals(educationCode, that.educationCode) &&
                Objects.equals(specialityCode, that.specialityCode) &&
                Objects.equals(specialityCodeSource, that.specialityCodeSource) &&
                Objects.equals(organizationId, that.organizationId) &&
                Objects.equals(organizationIdSource, that.organizationIdSource) &&
                Objects.equals(organizationName, that.organizationName) &&
                Objects.equals(userTitle, that.userTitle);
    }

    @Override
    public int hashCode() {

        return Objects.hash(serviceActorPID, personIdentifier, firstName, lastName, authorisationNumber, educationCode, specialityCode, specialityCodeSource, organizationId, organizationIdSource, organizationName, userTitle);
    }
}
