/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.dao;


import dk.sundhedsdatastyrelsen.livstestamente.domain.LivingWillVO;
import dk.sundhedsdatastyrelsen.livstestamente.exceptions.DAOException;

import java.time.Instant;
import java.util.LinkedList;
import java.util.List;

public interface LivingWillDAO {

    LivingWillVO getCurrentLivingWill(String personIdentifier, Instant validAt) throws DAOException;

    boolean existsLivingWillRecords(String personIdentifier) throws DAOException;

    long createLivingWill(LivingWillVO livingWillVO) throws DAOException;

    void updateValidToTimestamp(long pid, Instant newValidTo) throws DAOException;

    void deleteLivingWillRecord(String personIdentifier) throws DAOException;

    void deleteLivingWillRecords(List<String> personIdentifier) throws DAOException;

    LinkedList<String> getAllPersonsFromLTR() throws DAOException;

    LinkedList<String> getAllPersonsFromLTR(int limit) throws DAOException;

    LinkedList<String> getAllPersonsFromLTRWithPrefix(String prefix) throws DAOException;
}
