/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.dao.impl;


import dk.sundhedsdatastyrelsen.livstestamente.dao.impl.abstr.VersionedAbstractRowMapper;
import dk.sundhedsdatastyrelsen.livstestamente.domain.AcceptanceNeeded;
import dk.sundhedsdatastyrelsen.livstestamente.domain.TreatmentWillVO;
import org.springframework.jdbc.core.RowMapper;

import java.sql.ResultSet;
import java.sql.SQLException;

public class TreatmentWillRowMapper extends VersionedAbstractRowMapper<TreatmentWillVO> implements RowMapper<TreatmentWillVO> {

    @Override
    public TreatmentWillVO mapRow(ResultSet resultSet, int i) throws SQLException {
        TreatmentWillVO treatmentWillVO = new TreatmentWillVO();
        treatmentWillVO.setTreatmentWillPID(resultSet.getLong("TreatmentWillPID"));
        treatmentWillVO.setPersonIdentifier(resultSet.getString("PersonIdentifier"));

        treatmentWillVO.setNoLifeProlongingIfDying(resultSet.getBoolean("NoLifeProlongingIfDying"));
        treatmentWillVO.setNoLifeProlongingIfDyingAcceptanceNeeded(
                AcceptanceNeeded.fromDatabaseValue(
                        mapStringJavaNull("NoLifeProlongingIfDyingAcceptanceNeeded", resultSet)));

        treatmentWillVO.setNoLifeProlongingIfSeverelyDegraded(resultSet.getBoolean("NoLifeProlongingIfSeverelyDegraded"));
        treatmentWillVO.setNoLifeProlongingIfSeverelyDegradedAcceptanceNeeded(
                AcceptanceNeeded.fromDatabaseValue(
                        mapStringJavaNull("NoLifeProlongingIfSeverelyDegradedAcceptanceNeeded", resultSet)));

        treatmentWillVO.setNoLifeProlongingIfSeverePain(resultSet.getBoolean("NoLifeProlongingIfSeverePain"));
        treatmentWillVO.setNoLifeProlongingIfSeverePainAcceptanceNeeded(
                AcceptanceNeeded.fromDatabaseValue(
                        mapStringJavaNull("NoLifeProlongingIfSeverePainAcceptanceNeeded", resultSet)));

        treatmentWillVO.setNoForcedTreatmentIfIncapable(resultSet.getBoolean("NoForcedTreatmentIfIncapable"));
        treatmentWillVO.setNoForcedTreatmentIfIncapableAcceptanceNeeded(
                AcceptanceNeeded.fromDatabaseValue(
                        mapStringJavaNull("NoForcedTreatmentIfIncapableAcceptanceNeeded", resultSet)));

        super.mapRowFromVersionData(treatmentWillVO, resultSet);
        return treatmentWillVO;
    }

    private String mapStringJavaNull(String columnLabel, ResultSet resultSet) throws SQLException {
        final String s = resultSet.getString(columnLabel);
        return resultSet.wasNull() ? null : s;
    }

}
