/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.dao.impl.abstr;

import dk.sundhedsdatastyrelsen.livstestamente.dao.PropertiesDAO;
import dk.sundhedsdatastyrelsen.livstestamente.dao.impl.PropertiesRowMapper;
import dk.sundhedsdatastyrelsen.livstestamente.domain.PropertiesVO;
import dk.sundhedsdatastyrelsen.livstestamente.exceptions.DAOException;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

import javax.sql.DataSource;
import java.sql.Types;

public abstract class AbstractPropertiesDAO extends AbstractDAOImpl implements PropertiesDAO {

    private final String tableName;

    protected AbstractPropertiesDAO(DataSource dataSource, String tableName) {
        super(dataSource);
        this.tableName = tableName;
    }

    @Override
    public PropertiesVO getProperty(String key) throws DAOException {
        try {
            final String sql = "SELECT * FROM " + tableName + " " +
                    "WHERE PropertyKey = :propertyKey";
            final MapSqlParameterSource parameterSource = new MapSqlParameterSource()
                    .addValue("propertyKey", key, Types.VARCHAR);
            return DataAccessUtils.singleResult(namedParamJdbcTemplate.query(sql, parameterSource, new PropertiesRowMapper()));
        } catch (DataAccessException e) {
            throw new DAOException("An error occurred reading property with PropertyKey: " + key, e);
        }
    }

    @Override
    public void updateProperty(String key, String value) throws DAOException {
        try {
            final PropertiesVO existingProperty = getProperty(key);
            String sql;
            if (existingProperty != null) {
                sql = "UPDATE " + tableName + " SET PropertyValue = :propertyValue " +
                        "WHERE PropertyKey = :propertyKey";
            } else {
                sql = "INSERT INTO " + tableName + " (PropertyKey, PropertyValue) " +
                        "VALUES (:propertyKey, :propertyValue)";
            }
            final MapSqlParameterSource parameters = new MapSqlParameterSource()
                    .addValue("propertyKey", key, Types.VARCHAR)
                    .addValue("propertyValue", value, Types.VARCHAR);
            namedParamJdbcTemplate.update(sql, parameters);
        } catch (DataAccessException e) {
            throw new DAOException("An error occurred updating property with PropertyKey: " + key + ". PropertyValue: " + value, e);
        }
    }
}
