/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.util;

public enum FaultCode {
    GENERAL_ERROR(100, "Der opstod en fejl", false),
    AUTHORIZATION_ERROR_INVALID_HEADER(101, "Sikkerhedsbillet er ikke gyldige", true),
    AUTHORIZATION_ERROR_INGEN_DELEGERING(101, "Delegering ikke tilladt", true),
    AUTHORIZATION_ERROR_INGEN_CVR(101, "CVR nummer ikke fundet i sikkerhedsbilletten", true),
    AUTHORIZATION_ERROR_USER_NOT_CPR(101, "Bruger skal være identificeret ved CPR nummer", true),
    AUTHORIZATION_ERROR_USERTYPE_NOT_SPECIFIED(101, "Brugertype kunne ikke bestemmes udfra sikkerhedsbillet", true),
    AUTHORIZATION_ERROR_USERTYPE_ADMIN_NOT_ALLOWED(101, "Sikkerhedsbillet giver ikke adgang til admin rollen", true),
    AUTHORIZATION_ERROR_WRONG_AUDIENCE(101, "Forkert audience i sikkerhedsbillet", true),
    AUTHORIZATION_ERROR_CVR_NOT_WHITELISTED(101, "CVR nummer ikke whitelisted til kald for systembrugere", true),
    AUTHORIZATION_ERROR_PERSON_IDENTIFIER_ACCESS_DENIED(102, "Adgang til CPR-nummeret ikke tilladt", true),
    
    CREATE_NOT_ALLOWED(103, "Det er ikke tilladt at oprette registreringer", true),

    // Living Will
    LIVING_WILL_NO_DATA_FOUND(300, "Ingen data fundet for Livstestamente", true),
    LIVING_WILL_ALREADY_EXISTING(301, "Der eksisterer allerede data for Livstestamente", true),

    GET_LIVING_WILL_VALIDATION_ERROR(200, "Fejl i request i forbindelse med hent Livstestamente", true),
    GET_LIVING_WILL_INTERNAL_SERVER_ERROR(201, "Intern fejl fejl i forbindelse med hent Livstestamente", false),

    HAS_LIVING_WILL_VALIDATION_ERROR(210, "Fejl i request i forbindelse med hent status for Livstestamente", true),
    HAS_LIVING_WILL_INTERNAL_SERVER_ERROR(211, "Intern fejl i forbindelse med hent status for Livstestamente", false),

    CREATE_LIVING_WILL_VALIDATION_ERROR(220, "Fejl i request i forbindelse med oprettelse af Livstestamente", true),
    CREATE_LIVING_WILL_INTERNAL_SERVER_ERROR(221, "Intern fejl i forbindelse med oprettelse af Livstestamente", false),

    UPDATE_LIVING_WILL_VALIDATION_ERROR(230, "Fejl i request i forbindelse med opdatering af Livstestamente", true),
    UPDATE_LIVING_WILL_INTERNAL_SERVER_ERROR(431, "Intern fejl i forbindelse med opdatering af Livstestamente", false),

    DELETE_LIVING_WILL_VALIDATION_ERROR(240, "Fejl i request i forbindelse med sletning af Livstestamente", true),
    DELETE_LIVING_WILL_INTERNAL_SERVER_ERROR(241, "Intern fejl i forbindelse med sletning af Livstestamente", false),

    // Treatment Will
    TREATMENT_WILL_NO_DATA_FOUND(500, "Ingen data fundet for Behandlingstestamente", true),
    TREATMENT_WILL_ALREADY_EXISTING(501, "Der eksisterer allerede data for Behandlingstestamente", true),

    GET_TREATMENT_WILL_VALIDATION_ERROR(400, "Fejl i request i forbindelse med hent Behandlingstestamente", true),
    GET_TREATMENT_WILL_INTERNAL_SERVER_ERROR(401, "Intern fejl fejl i forbindelse med hent Behandlingstestamente", false),

    HAS_TREATMENT_WILL_VALIDATION_ERROR(410, "Fejl i request i forbindelse med hent status for Behandlingstestamente", true),
    HAS_TREATMENT_WILL_INTERNAL_SERVER_ERROR(411, "Intern fejl i forbindelse med hent status for Behandlingstestamente", false),

    CREATE_TREATMENT_WILL_VALIDATION_ERROR(420, "Fejl i request i forbindelse med oprettelse af Behandlingstestamente", true),
    CREATE_TREATMENT_WILL_INTERNAL_SERVER_ERROR(421, "Intern fejl i forbindelse med oprettelse af Behandlingstestamente", false),

    UPDATE_TREATMENT_WILL_VALIDATION_ERROR(430, "Fejl i request i forbindelse med opdatering af Behandlingstestamente", true),
    UPDATE_TREATMENT_WILL_INTERNAL_SERVER_ERROR(431, "Intern fejl i forbindelse med opdatering af Behandlingstestamente", false),

    DELETE_TREATMENT_WILL_VALIDATION_ERROR(440, "Fejl i request i forbindelse med sletning af Behandlingstestamente", true),
    DELETE_TREATMENT_WILL_INTERNAL_SERVER_ERROR(441, "Intern fejl i forbindelse med sletning af Behandlingstestamente", false);

    private int code;
    private String message;
    private boolean warningOnly;

    public int getCode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    public boolean isWarningOnly() {
        return warningOnly;
    }

    FaultCode(int code, String message, boolean warningOnly) {
        this.code = code;
        this.message = message;
        this.warningOnly = warningOnly;
    }

    @Override
    public String toString() {
        return code + ": " + message;
    }
}
