/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.util;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

/**
 * Throw fault from service
 */
public class FaultUtil {

    private static final Logger logger = LogManager.getLogger(FaultUtil.class);

    public static void throwFault(FaultCode faultCode) {
        throwFault(faultCode, null, null);
    }

    public static void throwFault(FaultCode faultCode, Throwable cause) {
        throwFault(faultCode, null, cause);
    }

    public static void throwFault(FaultCode faultCode, String faultInfo) {
        throwFault(faultCode, faultInfo, null);
    }

    public static void throwFault(FaultCode faultCode, String faultInfo, Throwable cause) {
        String message = faultCode.toString() + (faultInfo != null ? "; " + faultInfo : "");

        if (faultCode.isWarningOnly()) {
            logger.warn(message, cause);
        } else {
            logger.error(message, cause);
        }

        throw new ServiceException(faultCode, message, cause);
    }
}
