/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations.factory;

import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.joboperations.stack.livingwilldeceasedcleanup.LivingWillBatchCleanupOperation;
import dk.nsp.joboperations.stack.PersonInformationService;
import dk.nsp.joboperations.stack.livingwilldeceasedcleanup.LivingWillDeceasedPatientIdBatchCleanupSupplier;
import dk.nsp.joboperations.stack.livingwilldeceasedcleanup.LivingWillDeceasedPatientIdPrefixCleanupSupplier;
import dk.nsp.joboperations.stack.treatmentWilldeceasedcleanup.TreatmentWillDeceasedPatientIdBatchCleanupSupplier;
import dk.nsp.joboperations.stack.treatmentWilldeceasedcleanup.TreatmentWillDeceasedPatientIdPrefixCleanupSupplier;
import dk.nsp.joboperations.stack.treatmentWilldeceasedcleanup.TreatmentWillBatchCleanupOperation;
import dk.sundhedsdatastyrelsen.livstestamente.dao.LivingWillDAO;
import dk.sundhedsdatastyrelsen.livstestamente.dao.TreatmentWillDAO;

import java.time.Period;
import java.util.List;

public class CleanupOperationsFactory {
    private final LivingWillDAO livingWillDAO;
    private final TreatmentWillDAO treatmentWillDAO;

    private final DateTimeProvider dateTimeProvider;
    private final int batchSize;
    private final Period deceasedPeriod;
    private final PersonInformationService personInformationService;

    public CleanupOperationsFactory(LivingWillDAO livingWillDAO,
                                    TreatmentWillDAO treatmentWillDAO,
                                    DateTimeProvider dateTimeProvider,
                                    int batchSize,
                                    Period deceasedPeriod,
                                    PersonInformationService personInformationService) {
        this.livingWillDAO = livingWillDAO;
        this.treatmentWillDAO = treatmentWillDAO;
        this.dateTimeProvider = dateTimeProvider;
        this.batchSize = batchSize;
        this.deceasedPeriod = deceasedPeriod;
        this.personInformationService = personInformationService;
    }

    // LivingWill
    public LivingWillBatchCleanupOperation createLivingWillBatchCleanupOperation(List<String> cprEntriesForDeletion) {
        return new LivingWillBatchCleanupOperation(livingWillDAO, cprEntriesForDeletion, dateTimeProvider);
    }

    public LivingWillDeceasedPatientIdBatchCleanupSupplier createLivingWillDeceasedPatientIdBatchCleanupSupplier(List<String> cprs) {
        return new LivingWillDeceasedPatientIdBatchCleanupSupplier(cprs, dateTimeProvider, this, batchSize, deceasedPeriod, personInformationService);
    }

    public LivingWillDeceasedPatientIdPrefixCleanupSupplier createLivingWillDeceasedPatientIdPrefixCleanupSupplier(String patientIdPrefix) {
        return new LivingWillDeceasedPatientIdPrefixCleanupSupplier(livingWillDAO, this, batchSize, patientIdPrefix);
    }

    // TreatmentWill
    public TreatmentWillBatchCleanupOperation createTreatmentWillBatchCleanupOperation(List<String> cprEntriesForDeletion) {
        return new TreatmentWillBatchCleanupOperation(treatmentWillDAO, cprEntriesForDeletion, dateTimeProvider);
    }

    public TreatmentWillDeceasedPatientIdBatchCleanupSupplier createTreatmentWillDeceasedPatientIdBatchCleanupSupplier(List<String> cprs) {
        return new TreatmentWillDeceasedPatientIdBatchCleanupSupplier(cprs, dateTimeProvider, this, batchSize, deceasedPeriod, personInformationService);
    }

    public TreatmentWillDeceasedPatientIdPrefixCleanupSupplier createTreatmentWillDeceasedPatientIdPrefixCleanupSupplier(String patientIdPrefix) {
        return new TreatmentWillDeceasedPatientIdPrefixCleanupSupplier(treatmentWillDAO, this, batchSize, patientIdPrefix);
    }
}
