/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations.stack;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.backgroundjob.health.HealthIndicator;
import dk.nsp.backgroundjob.util.BackgroundJobException;
import dk.sundhedsdatastyrelsen.livstestamente.integrations.personInformation.PersonInformation;
import dk.sundhedsdatastyrelsen.livstestamente.integrations.personInformation.impl.PersonInformationServiceClientFactory;

import java.time.Instant;
import java.util.LinkedList;
import java.util.List;

public class PersonInformationService  implements HealthIndicator {

    private final PersonInformation personInformationClient;
    private final PersonInformationServiceClientFactory personInformationServiceClientFactory;

    public PersonInformationService(PersonInformation personInformationClient, PersonInformationServiceClientFactory personInformationServiceClientFactory) {
        this.personInformationClient = personInformationClient;
        this.personInformationServiceClientFactory = personInformationServiceClientFactory;
    }

    public List<String> filterToDeceased(List<String> cprs, Instant latestDeceasedTime)  {
        try {
            return personInformationClient.getPersonsDeceased(new LinkedList<>(cprs), latestDeceasedTime);
        } catch (Exception e) {
            throw new BackgroundJobException("Failed to call PatientInformation", e);
        }
    }

    @Override
    public HealthIndication getHealthInfo() {
        String serviceName = "PersonInformation";
        if (!personInformationServiceClientFactory.isHealthy()) {
            return new HealthIndication(500, "\"" + serviceName + "\": \"ERROR\"");
        } else {
            return new HealthIndication(200, "\"" + serviceName + "\": \"OK\"");
        }
    }
}
