/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations.stack.livingwilldeceasedcleanup;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.backgroundjob.util.BackgroundJobException;
import dk.sundhedsdatastyrelsen.livstestamente.dao.LivingWillDAO;
import org.apache.log4j.Logger;

import java.time.Duration;
import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;

public class LivingWillBatchCleanupOperation implements StackOperation {
    private final Logger log = Logger.getLogger(LivingWillBatchCleanupOperation.class);
    private final LivingWillDAO livingWillDAO;
    private final List<String> cprEntriesForDeletion;
    private final DateTimeProvider dateTimeProvider;
    private String operationName = "LivingWill Batch Cleanup";

    public LivingWillBatchCleanupOperation(LivingWillDAO livingWillDAO,
                                           List<String> cprEntriesForDeletion,
                                           DateTimeProvider dateTimeProvider) {
        this.livingWillDAO = livingWillDAO;
        this.cprEntriesForDeletion = cprEntriesForDeletion;
        this.dateTimeProvider = dateTimeProvider;
    }

    @Override
    public List<StackOperation> run() {
        log.trace(String.format("%s: Starting batch deletion of %d entries.", operationName, cprEntriesForDeletion.size()));
        ZonedDateTime startTime = dateTimeProvider.now();

        try {
            livingWillDAO.deleteLivingWillRecords(cprEntriesForDeletion);
        } catch (Exception e) {
            throw new BackgroundJobException("Failed to delete CPR entries", e);
        }

        log.info(String.format("%s: LivingWill batch cleanup finished! deleted=%d Time spent: %d seconds.", operationName, cprEntriesForDeletion.size(), Duration.between(startTime, dateTimeProvider.now()).getSeconds()));
        return Collections.emptyList();
    }

    public LivingWillBatchCleanupOperation withOperationName(String operationName) {
        this.operationName = operationName;
        return this;
    }
}
