/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations.stack.treatmentWilldeceasedcleanup;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.BackgroundJobException;
import dk.nsp.backgroundjob.util.BackgroundJobUtils;
import dk.nsp.joboperations.factory.CleanupOperationsFactory;
import dk.sundhedsdatastyrelsen.livstestamente.dao.TreatmentWillDAO;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class TreatmentWillDeceasedPatientIdPrefixCleanupSupplier implements StackOperation {
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final int batchSize;
    private final String patientIdPrefix;
    private final TreatmentWillDAO treatmentWillDAO;

    public TreatmentWillDeceasedPatientIdPrefixCleanupSupplier(TreatmentWillDAO treatmentWillDAO,
                                                               CleanupOperationsFactory cleanupOperationsFactory,
                                                               int batchSize,
                                                               String patientIdPrefix) {
        this.treatmentWillDAO = treatmentWillDAO;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.batchSize = batchSize;
        this.patientIdPrefix = patientIdPrefix;
    }

    @Override
    public List<StackOperation> run() {
        List<String> patientIds = getDocumentEntryPatientIdsWithPrefix();
        Collection<List<String>> patientIdBatches = BackgroundJobUtils.createShuffledBatches(patientIds, batchSize);
        return patientIdBatches.stream()
                .map(cleanupOperationsFactory::createTreatmentWillDeceasedPatientIdBatchCleanupSupplier)
                .collect(Collectors.toList());
    }

    private List<String> getDocumentEntryPatientIdsWithPrefix() {
        try {
            return treatmentWillDAO.getAllPersonsFromLTRWithPrefix(patientIdPrefix);
        } catch (Exception e) {
            throw new BackgroundJobException("Failed to retrieve CPR entry using prefix " + patientIdPrefix, e);
        }
    }
}
