/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations.stack;

import dk.nsp.backgroundjob.util.BackgroundJobException;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.joboperations.stack.livingwilldeceasedcleanup.LivingWillBatchCleanupOperation;
import dk.sundhedsdatastyrelsen.livstestamente.dao.LivingWillDAO;
import dk.sundhedsdatastyrelsen.livstestamente.exceptions.DAOException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.LinkedList;

import static dk.nsp.joboperations.stack.TestUtils.assertThrows;
import static org.junit.Assert.assertTrue;

public class LivingWillBatchCleanupOperationTest {

    private LivingWillDAO livingWillDAO;
    private LinkedList<String> cprEntriesForDeletion;
    private LivingWillBatchCleanupOperation livingWillBatchCleanupOperation;

    @Before
    public void setUp() throws Exception {
        livingWillDAO = Mockito.mock(LivingWillDAO.class);
        cprEntriesForDeletion = new LinkedList<>();
        cprEntriesForDeletion.add("1");
        livingWillBatchCleanupOperation = new LivingWillBatchCleanupOperation(livingWillDAO, cprEntriesForDeletion, new DateTimeProvider());
    }

    @Test
    public void run_WhenDeletionSucceeds_DeletesDocumentEntries() throws DAOException {
        livingWillBatchCleanupOperation.run();

        Mockito.verify(livingWillDAO).deleteLivingWillRecords(cprEntriesForDeletion);
    }

    @Test
    public void run_WhenDeletionFails_ThrowsCleanupException() throws DAOException {
        Mockito.doThrow(RuntimeException.class).when(livingWillDAO).deleteLivingWillRecords(cprEntriesForDeletion);

        BackgroundJobException e = assertThrows(BackgroundJobException.class, () -> livingWillBatchCleanupOperation.run());
        assertTrue(e.getMessage(), e.getMessage().contains("Failed to delete CPR entries"));
    }
}