/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations.stack.livingwilldeceasedcleanup;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.joboperations.factory.CleanupOperationsFactory;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;

public class LivingWillDeceasedCleanupSupplierTest {

    private CleanupOperationsFactory cleanupOperationsFactory;
    private LivingWillDeceasedCleanupSupplier livingWillDeceasedCleanupSupplier;

    @Before
    public void setUp() {
        cleanupOperationsFactory = Mockito.mock(CleanupOperationsFactory.class);
        livingWillDeceasedCleanupSupplier = new LivingWillDeceasedCleanupSupplier(cleanupOperationsFactory);
    }

    @Test
    public void run_Returns10ExpectedSuppliers() {
        List<LivingWillDeceasedPatientIdPrefixCleanupSupplier> prefixSuppliers = new ArrayList<>();
        for (int i = 0; i < 100; i++) {
            prefixSuppliers.add(Mockito.mock(LivingWillDeceasedPatientIdPrefixCleanupSupplier.class));
        }

        Mockito.when(cleanupOperationsFactory.createLivingWillDeceasedPatientIdPrefixCleanupSupplier(Mockito.anyString()))
                .thenReturn(prefixSuppliers.get(0))
                .thenReturn(prefixSuppliers.get(1))
                .thenReturn(prefixSuppliers.get(2))
                .thenReturn(prefixSuppliers.get(3))
                .thenReturn(prefixSuppliers.get(4))
                .thenReturn(prefixSuppliers.get(95))
                .thenReturn(prefixSuppliers.get(96))
                .thenReturn(prefixSuppliers.get(97))
                .thenReturn(prefixSuppliers.get(98))
                .thenReturn(prefixSuppliers.get(99));

        List<StackOperation> stackOperations = livingWillDeceasedCleanupSupplier.run();
        assertEquals(100, stackOperations.size());

        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("00");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("01");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("02");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("03");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("04");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("95");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("96");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("97");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("98");
        Mockito.verify(cleanupOperationsFactory).createLivingWillDeceasedPatientIdPrefixCleanupSupplier("99");
    }
}