/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.idptest;

import dk.nsp.test.idp.NspSoapClient;
import dk.nsp.test.idp.model.Identity;
import org.apache.commons.io.IOUtils;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.net.URI;
import java.nio.charset.StandardCharsets;

public class ServiceCaller {

    public <RequestType, ResponseType> ResponseType call(Identity identity, RequestType request, JAXBContext jaxbContext, String action, URI uri, Class responseClass) throws JAXBException, NspServiceException {
        Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
        StringWriter sw = new StringWriter();
        jaxbMarshaller.marshal(request, sw);

        try(NspSoapClient client = new NspSoapClient(this::log)) {
            NspSoapClient.NspSoapRequest soapRequest = null;
            JAXBElement<ResponseType> jaxbElement;

            if (identity != null) {
                soapRequest = client.request(uri, action).as(identity);
            }

            try (NspSoapClient.NspSoapResponse soapResponse = soapRequest.execute(sw.toString())) {
                String responseXml = IOUtils.toString(soapResponse.getResponse(), StandardCharsets.UTF_8);

                if (soapResponse.isFault()) {
                    throw new NspServiceException(responseXml);
                }

                SOAPMessage message = MessageFactory.newInstance().createMessage(null, new ByteArrayInputStream(responseXml.getBytes()));
                jaxbElement = (JAXBElement<ResponseType>) jaxbContext.createBinder().unmarshal(message.getSOAPBody().extractContentAsDocument(), responseClass);

                return (ResponseType) jaxbElement.getValue();

            } catch (NspServiceException e) {
                throw e;
            }
        } catch (Exception e){
            throw new RuntimeException(e);
        }
    }

    private void log(String xml) {
        if (Boolean.getBoolean(this.getClass().getSimpleName() + ".log")) {
            String method = "Unknown";
            for (StackTraceElement e : new Exception().fillInStackTrace().getStackTrace()) {
                if (e.getClassName().equals(getClass().getName())) {
                    method = e.getMethodName();
                }
            }
            System.out.println("[" + method + "]");
            System.out.println(xml.replace("\n", ""));
        }
    }
}
