/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations;

import dk.nsp.joboperations.util.BtrFacade;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.junit.Before;

import java.io.IOException;

public class AbstractITCase {

    public BtrFacade btrFacade;
    public String btrEndpoint;
    public String host = "localhost";
    public String port = "8080";
    public String portToBtrFacade = "8080";

    // Test personen er i test1 og i initiailzerJson.Json filen
    public static final String TEST_DEAD_PERSON = "0810018631";

    @Before
    public void setup() {
    }

    public AbstractITCase() {
        String host = System.getProperty("BTR_TEST_HOST");
        String port = System.getProperty("BTR_TEST_PORT");

        if (host != null && host.length() > 0) {
            this.host = host;
        }

        if (port != null && port.length() > 0) {
            this.port = port;
        }

        // Localhost bruger port 8081 for btr-job-operations, som kan ses i docker compose filen
        boolean isLocalhost = this.host.equals("localhost") || this.host.equals("127.0.0.1");
        if (!this.port.equals("8081") && isLocalhost) {
            this.port = "8081";
        }

        // Localhost bruger port 8080 for btr, som kan ses i docker compose filen
        if (!this.port.equals("8080") && isLocalhost) {
            this.portToBtrFacade = "8080";
        }

        btrEndpoint = "http://" + this.host + ":" + this.port + "/btr";
        btrFacade = new BtrFacade("http://" + this.host + ":" + this.portToBtrFacade + "/btr");
    }

    protected int btrDeceasedCleanupCall(String endpoint) throws IOException, InterruptedException {
        try (CloseableHttpClient httpClient = HttpClients.createDefault()) {
            HttpGet request = new HttpGet(endpoint);
            try (CloseableHttpResponse response = httpClient.execute(request)) {
                return response.getStatusLine().getStatusCode();
            }
        }
    }
}
