/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.joboperations;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.net.URISyntaxException;

public class TreatmentWillDeceasedCleanupITCase extends AbstractITCase {

    private String startEndpoint;
    private String statusEndpoint;

    public TreatmentWillDeceasedCleanupITCase() throws URISyntaxException {
        super();
        this.startEndpoint = this.btrEndpoint + "/treatment-will-deceased-cleanup/start";
        this.statusEndpoint = this.btrEndpoint + "/treatment-will-deceased-cleanup/status";
    }

    @Before
    public void setUp() throws Exception {
        // personinformation deceased er mock i initializerJson.json til kun at returner TEST_DEAD_PERSON
        boolean hasTreatmentWill = btrFacade.hasTreatmentWill(TEST_DEAD_PERSON);
        if (!hasTreatmentWill) {
            btrFacade.createTreatmentWill(TEST_DEAD_PERSON);
            hasTreatmentWill = btrFacade.hasTreatmentWill(TEST_DEAD_PERSON);
            Assert.assertTrue(hasTreatmentWill);
        }
    }

    @Test
    public void callStatusUrlSuccess() throws Exception {
        // given

        // when
        int responseCode = btrDeceasedCleanupCall(statusEndpoint);

        // then
        Assert.assertEquals("Assert fail on address: " + statusEndpoint, 200, responseCode);
    }

    @Test
    public void callStartUrlSuccess() throws Exception {
        // given
        boolean hasTreatmentWill = btrFacade.hasTreatmentWill(TEST_DEAD_PERSON);
        Assert.assertTrue("Should have a treatment will", hasTreatmentWill);

        // when
        int responseCode = btrDeceasedCleanupCall(startEndpoint);

        // then
        Assert.assertEquals("Assert fail on address: " + startEndpoint, 200, responseCode);
        hasTreatmentWill = btrFacade.hasTreatmentWill(TEST_DEAD_PERSON);
        Assert.assertFalse("Should not have a treatment will", hasTreatmentWill);
    }
}
