/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.wildfly;

import dk.sundhedsdatastyrelsen.livstestamente._2018._05._01.HasLivingWillRequest;
import dk.sundhedsdatastyrelsen.livstestamente.exceptions.ValidationException;
import dk.sundhedsdatastyrelsen.livstestamente.service.webservice.hl7cda.LivingWillHL7CDAValidator;
import dk.sundhedsdatastyrelsen.livstestamente.integrations.personInformation.PersonInformation;
import org.hl7.v3.II;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.mock;

public class LivingWillTest {

    @Test
    public void livingWillHL7CDAValidator_ageTooLow_errorShouldBeReturned() throws Exception {

        final String expectedError = "Age validation failed";
        PersonInformation personInformation = mock(PersonInformation.class);
        doThrow(new ValidationException(expectedError)).when(personInformation).validateAge(anyString());

        II ii = new II(); ii.setRoot("1.2.208.176.1.2"); ii.setExtension("1234567890");
        ii.setAssigningAuthorityName("CPR");
        HasLivingWillRequest request = new HasLivingWillRequest(); request.setId(ii);
        IllegalArgumentException ex = assertThrows(IllegalArgumentException.class,
                () -> new LivingWillHL7CDAValidator(personInformation).validate(request));
        assertEquals(expectedError, ex.getMessage());
    }
}
