/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.wildfly;

import dk.nsp.test.idp.EmployeeIdentities;
import dk.nsp.test.idp.model.EmployeeIdentity;
import dk.sundhedsdatastyrelsen.livstestamente._2018._05._01.*;
import dk.idptest.SOAPOperation20180501;
import org.junit.Assume;
import org.junit.Before;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.File;
import java.net.URI;

import static org.junit.Assert.assertNotNull;

/**
 * IDWS integration tests
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class LivingWillV2DGWSITCase extends AbstractITCase {
    private static final String REQUEST_FILE_PATH = "/dgws/testfiles/2020.03.16/";

    private String uri;
    private Unmarshaller unmarshaller;
    private EmployeeIdentity actor = EmployeeIdentities.lægeCharlesBabbage();

    @Before
    public void before() throws JAXBException {
        uri = super.getItAddress() + "/ltrAdmin";
        if(uri.startsWith("null")) {
            uri = "http://localhost:8080/btr" + "/ltrAdmin";
        }
        setUpMarshalling();
    }

    private void setUpMarshalling() throws JAXBException {
        JAXBContext jaxbContext = JAXBContext.newInstance(ObjectFactory.class);
        unmarshaller = jaxbContext.createUnmarshaller();
    }

    @Test
    public void step1_DGWSRequest_UpdateLivingWill() throws Exception {
        TreatmentWillV2DGWSITCase treatmentWillV2DGWSITCase = new TreatmentWillV2DGWSITCase();
        treatmentWillV2DGWSITCase.before();

        Assume.assumeTrue(
              DGWSRequest_HasLivingWill("HasLivingWill_request_body_0309691444.xml") &&
                !treatmentWillV2DGWSITCase.DGWSRequest_HasTreatmentWill("HasTreatmentWill_request_body_0309691444.xml"));

        JAXBContext jaxbContext = JAXBContext.newInstance(UpdateLivingWillRequest.class, UpdateLivingWillResponse.class);

        File requestFile = getRequestFile(REQUEST_FILE_PATH + "UpdateLivingWill_request_body.xml");
        UpdateLivingWillRequest request = (UpdateLivingWillRequest) unmarshaller.unmarshal(requestFile);
        UpdateLivingWillResponse response = serviceCaller.call(
                actor,
                request,
                jaxbContext,
                SOAPOperation20180501.UpdateLivingWill.getSoapAction(),
                new URI(uri),
                UpdateLivingWillResponse.class);

        // The response is empty, but should not be null
        assertNotNull(response);
    }

    public boolean DGWSRequest_HasLivingWill(String requestFileName) throws Exception {
        JAXBContext jaxbContext = JAXBContext.newInstance(HasLivingWillRequest.class, HasLivingWillResponse.class);

        File requestFile = getRequestFile(REQUEST_FILE_PATH + requestFileName);
        HasLivingWillRequest request = (HasLivingWillRequest) unmarshaller.unmarshal(requestFile);
        HasLivingWillResponse response = serviceCaller.call(
                actor,
                request,
                jaxbContext,
                SOAPOperation20180501.HasLivingWill.getSoapAction(),
                new URI(uri),
                HasLivingWillResponse.class);

        // The response is empty, but should not be null
        assertNotNull(response);
        return response.isWillExists();
    }
}
