/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.wildfly;

import dk.nsp.test.idp.builder.Builders;
import dk.nsp.test.idp.model.CitizenIdentity;
import dk.nsp.test.idp.vault.RemoteCredentialVault;
import dk.sundhedsdatastyrelsen.livstestamente._2018._05._01.*;
import dk.idptest.SOAPOperation20180501;
import org.junit.Assume;
import org.junit.Before;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.File;
import java.net.URI;

import static org.junit.Assert.assertNotNull;

/**
 * IDWS integration tests
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class LivingWillV2IDWSITCase extends AbstractITCase {
    private static final String REQUEST_FILE_PATH = "/idws/testfiles/2020.03.16/";
    private static final String RESPONSE_AUDIENCE_URI = "https://fsk";
    private String uri;
    private Unmarshaller unmarshaller;
    private CitizenIdentity actor = Builders.citizenIdentity().representing(Builders.citizen().identifiedBy("0309691444")).usedBy(Builders.serviceConsumer().named("Service Consumer Test").identifiedBy(new RemoteCredentialVault(URI.create("https://www.nspop.dk/download/attachments/190481050/NSP_Test_Service_Consumer_sds.p12"), "Test1234".toCharArray(), true))).verifiedBy(Builders.identityProvider().named("https://idp.test.nspop.dk").withAudience("https://bootstrap.sts.nspop.dk/").identifiedBy(new RemoteCredentialVault(URI.create("https://www.nspop.dk/download/attachments/190481050/NSP_Test_Identity_Provider_sds.p12"), "Test1234".toCharArray(), true))).using(Builders.securityTokenService().at(URI.create("http://test1.ekstern-test.nspop.dk:8080/sts/services/Bst2Idws"))).consumedBy(Builders.serviceProvider().withAudience(RESPONSE_AUDIENCE_URI)).build();

    @Before
    public void before() throws JAXBException {
        uri = super.getItAddress() + "/ltr";
        if(uri.startsWith("null")) {
            uri = "http://localhost:8080/btr" + "/ltr";
        }
        setUpMarshalling();
    }

    private void setUpMarshalling() throws JAXBException {
        JAXBContext jaxbContext = JAXBContext.newInstance(ObjectFactory.class);
        unmarshaller = jaxbContext.createUnmarshaller();
    }

    @Test
    public void step1_IDWSRequest_UpdateLivingWill() throws Exception {
        LivingWillV2DGWSITCase livingWillV2DGWSITCase = new LivingWillV2DGWSITCase();
        livingWillV2DGWSITCase.before();
        TreatmentWillV2DGWSITCase treatmentWillV2DGWSITCase = new TreatmentWillV2DGWSITCase();
        treatmentWillV2DGWSITCase.before();

        Assume.assumeTrue(
                livingWillV2DGWSITCase.DGWSRequest_HasLivingWill("HasLivingWill_request_body_0309691444.xml") &&
              !treatmentWillV2DGWSITCase.DGWSRequest_HasTreatmentWill( "HasTreatmentWill_request_body_0309691444.xml"));

        JAXBContext jaxbContext = JAXBContext.newInstance(UpdateLivingWillRequest.class, UpdateLivingWillResponse.class);

        File requestFile = getRequestFile(REQUEST_FILE_PATH + "UpdateLivingWill_request_body.xml");
        UpdateLivingWillRequest request = (UpdateLivingWillRequest) unmarshaller.unmarshal(requestFile);
        UpdateLivingWillResponse response = serviceCaller.call(
                actor,
                request,
                jaxbContext,
                SOAPOperation20180501.UpdateLivingWill.getSoapAction(),
                new URI(uri),
                UpdateLivingWillResponse.class);

        // The response is empty, but should not be null
        assertNotNull(response);
    }
}
