/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service;

import dk.sundhedsdatastyrelsen.livstestamente.util.HealthIndicator;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.CharArrayWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;

import static org.apache.http.HttpStatus.SC_INTERNAL_SERVER_ERROR;
import static org.apache.http.HttpStatus.SC_OK;

@WebServlet(name="AlarmServlet", urlPatterns={"/alarm"})
public class AlarmServlet extends AutowiredHttpServlet{
    @Autowired(required = false)
    private Collection<HealthIndicator> healthIndicators;

    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
        int status = SC_OK;

        final CharArrayWriter caw = new CharArrayWriter();
        if (healthIndicators != null) {
            try(PrintWriter pw = new PrintWriter(caw)) {
                for (HealthIndicator healthIndicator : healthIndicators) {
                    if (healthIndicator != null) {
                        int indicatorStatus = healthIndicator.getStatusCode();
                        if (indicatorStatus != SC_OK) {
                            status = SC_INTERNAL_SERVER_ERROR;
                            String statusMessage = " Non-Authoritative Information from: " + healthIndicator.getName();
                            pw.println(String.format("%s %s", indicatorStatus, statusMessage));
                            pw.println(healthIndicator.getStatusText());
                        }
                    }
                }
            }
        }

        response.setStatus(status);
        if(status != SC_OK) {
            response.setContentType("text/plain");
            caw.writeTo(response.getWriter());
        }
    }
}
