/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.conf;

import dk.sdsd.nsp.slalog.api.SLALogConfig;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.pki.SOSIFederation;
import dk.sosi.seal.pki.SOSITestFederation;
import dk.sosi.seal.vault.CredentialVault;
import dk.sundhedsdatastyrelsen.livingwill._2022._08._01.LivingWillUpdated;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.NasFacade;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.NasFacadeBuilder;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dgws.*;
import dk.sundhedsdatastyrelsen.livstestamente.conf.Properties;
import dk.sundhedsdatastyrelsen.treatmentwill._2022._08._01.TreatmentWillUpdated;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class NasConfig {

    @Autowired
    Properties properties;

    @Autowired
    private CredentialVaultFactory credentialVaultFactory;

    @Bean ("TreatmentWillNasFacade")
    public NasFacade nasTreatmentWillFacade(Properties properties) throws StsException {
        SLALogConfig slaLogConfig = new SLALogConfig(properties.getAppName(), properties.getAppShortName());
        int failThreshold = Integer.parseInt(properties.getNasFailThreshold());
        int connectTimeoutMillis = Integer.parseInt(properties.getNasConnectTimeoutMillis());
        int readTimeoutMillis = Integer.parseInt(properties.getNasReadTimeoutMillis());
        int maxTotalConnections = Integer.parseInt(properties.getNasMaxTotalConnections());
        int defaultMaxConnectionsPerRoute = Integer.parseInt(properties.getNasDefaultMaxConnectionsPerRoute());
        String nasServiceUrl = properties.getNasServiceUrl();

        int maxProxyPoolSize = 20;

        return new NasFacadeBuilder(slaLogConfig, failThreshold, connectTimeoutMillis, readTimeoutMillis, maxTotalConnections, defaultMaxConnectionsPerRoute, nasServiceUrl, TreatmentWillUpdated.class)
                .setMaxProxyPoolSize(maxProxyPoolSize)
                .withDgwsHeaderFactory(getDgwsHeaderFactory(properties))
                .build();
    }

    @Bean("LivingWillNasFacade")
    public NasFacade nasLivingWillFacade(Properties properties) throws StsException {
        SLALogConfig slaLogConfig = new SLALogConfig(properties.getAppName(), properties.getAppShortName());
        int failThreshold = Integer.parseInt(properties.getNasFailThreshold());
        int connectTimeoutMillis = Integer.parseInt(properties.getNasConnectTimeoutMillis());
        int readTimeoutMillis = Integer.parseInt(properties.getNasReadTimeoutMillis());
        int maxTotalConnections = Integer.parseInt(properties.getNasMaxTotalConnections());
        int defaultMaxConnectionsPerRoute = Integer.parseInt(properties.getNasDefaultMaxConnectionsPerRoute());
        String nasServiceUrl = properties.getNasServiceUrl();

        int maxProxyPoolSize = 20;

        // use superclass?
        return new NasFacadeBuilder(slaLogConfig, failThreshold, connectTimeoutMillis, readTimeoutMillis, maxTotalConnections, defaultMaxConnectionsPerRoute, nasServiceUrl, LivingWillUpdated.class)
                .setMaxProxyPoolSize(maxProxyPoolSize)
                .withDgwsHeaderFactory(getDgwsHeaderFactory(properties))
                .build();
    }

    private DgwsHeaderFactory getDgwsHeaderFactory(Properties properties) throws StsException {
        java.util.Properties sosiProperties = new java.util.Properties();
        Federation federation = null;
        if (!properties.getStsTestMode()) {
            federation = new SOSIFederation(sosiProperties);
        } else {
            federation = new SOSITestFederation(sosiProperties);
        }

        CredentialVault credentialVault = credentialVaultFactory.getCredentilVaultFromProperties(properties, sosiProperties);
        SOSIFactory sosiFactory = new SOSIFactory(federation, credentialVault, sosiProperties);

        String stsEndpoint = properties.getStsEndpoint();
        StsClient stsClient = new StsClientImpl(sosiFactory, stsEndpoint);

        String itSystem = properties.getIdcardSystemName();
        String subjectType = properties.getIdcardSubjectIdType();
        String subjectId = properties.getIdcardSubjectId();
        String subjectName = properties.getIdcardSubjectName();

        DgwsHeaderFactory dgwsHeaderFactory = new NspServiceDgwsHeaderFactoryImpl(sosiFactory, stsClient, itSystem, itSystem, subjectType, subjectId, subjectName);

        // Trigger issueing of IDcard during startup
        dgwsHeaderFactory.getDgwsHeaders("testOnStartup");

        return dgwsHeaderFactory;
    }
}
