/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.context;

import java.time.Instant;
import java.util.UUID;

import dk.sundhedsdatastyrelsen.livstestamente.context.ServiceActor;
import dk.sundhedsdatastyrelsen.livstestamente.context.ServiceActorProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import dk.sds.nsp.security.SecurityContext;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sundhedsdatastyrelsen.livstestamente.domain.RequestContextVO;

@Component
public class RequestContextProvider {

	private ServiceActorProvider serviceActorProvider;
	
	private SecurityContextProvider securityContextProvider;
	
	public RequestContextProvider(@Autowired SecurityContextProvider securityContextProvider, @Autowired ServiceActorProvider serviceActorProvider) {
		this.serviceActorProvider = serviceActorProvider;
		this.securityContextProvider = securityContextProvider;
	}
	
	public RequestContextVO getRequestContext(String patientIdentifier, boolean wantsAdmin, SLALogItem slaLogItem) {
		
		SecurityContext sc =  securityContextProvider.getSecurityContext();
		
		ServiceActor serviceActor = serviceActorProvider.getServiceActor(sc, wantsAdmin);
		
        RequestContextVO context = new RequestContextVO(serviceActor, slaLogItem);
        context.setTime(Instant.now()); // Get current time for request
        context.setPersonIdentifier(patientIdentifier);
        
        if (sc.getMessage().isPresent()) {
        	mapDataFromMessage(context, sc.getMessage().get());
        }
        
        if (slaLogItem != null && context.getMessageId() != null) {
            slaLogItem.setMessageId(context.getMessageId());
        }
        context.setSlaLogItem(slaLogItem);
        return context;

	}
	
	private void mapDataFromMessage(RequestContextVO requestContextVO, SecurityContext.Message message) {
		requestContextVO.setMessageId(message.getIdentifier().orElse(UUID.randomUUID().toString()));
		requestContextVO.setFlowId(message.getConversationIdentifier().orElse(UUID.randomUUID().toString()));
	}
}
