/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.minlog;

import java.time.Instant;
import java.util.Objects;

import dk.sds.nsp.security.SecurityContext.Organisation.OrganisationIdentifierFormat;

public class MinLogData {
    private String activity;
    private Instant eventDateTime;
    private String organizationId;
    private String organizationName;
    private OrganisationIdentifierFormat organizationType;
    private String citizenCpr;
    private String sessionId;
    private String userCPR;
    private String onBehalfOfCpr;
    private String sourceSystemIdentifier;
    private String userRole;

    public MinLogData(String activity, Instant eventDateTime, String organizationId, String organizationName, OrganisationIdentifierFormat organizationType, String citizenCpr, String sessionId, String userCPR, String onBehalfOfCpr, String sourceSystemIdentifier, String userRole) {
        this.activity = activity;
        this.eventDateTime = eventDateTime;
        this.organizationId = organizationId;
        this.organizationName = organizationName;
        this.organizationType = organizationType;
        this.citizenCpr = citizenCpr;
        this.sessionId = sessionId;
        this.userCPR = userCPR;
        this.onBehalfOfCpr = onBehalfOfCpr;
        this.sourceSystemIdentifier = sourceSystemIdentifier;
        this.userRole = userRole;

    }

    public String getActivity() {
        return activity;
    }

    public Instant getEventDateTime() {
        return eventDateTime;
    }

    public String getOrganizationId() {
        return organizationId;
    }

    public String getOrganizationName() {
        return organizationName;
    }

    public OrganisationIdentifierFormat getOrganizationType() {
        return organizationType;
    }

    public String getCitizenCpr() {
        return citizenCpr;
    }

    public String getSessionId() {
        return sessionId;
    }

    public String getUserCPR() {
        return userCPR;
    }

    public String getOnBehalfOfCpr() {
        return onBehalfOfCpr;
    }

    public String getSourceSystemIdentifier() {
        return sourceSystemIdentifier;
    }

    public String getUserRole() {
        return userRole;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        MinLogData that = (MinLogData) o;
        return Objects.equals(activity, that.activity) &&
                Objects.equals(eventDateTime, that.eventDateTime) &&
                Objects.equals(organizationId, that.organizationId) &&
                Objects.equals(organizationName, that.organizationName) &&
                organizationType == that.organizationType &&
                Objects.equals(citizenCpr, that.citizenCpr) &&
                Objects.equals(sessionId, that.sessionId) &&
                Objects.equals(userCPR, that.userCPR) &&
                Objects.equals(onBehalfOfCpr, that.onBehalfOfCpr) &&
                Objects.equals(sourceSystemIdentifier, that.sourceSystemIdentifier) &&
                Objects.equals(userRole, that.userRole);
    }

    @Override
    public int hashCode() {
        return Objects.hash(activity, eventDateTime, organizationId, organizationName, organizationType, citizenCpr, sessionId, userCPR, onBehalfOfCpr, sourceSystemIdentifier, userRole);
    }
}
