/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.minlog;

import java.time.Instant;
import java.time.temporal.ChronoUnit;

import org.springframework.stereotype.Service;

import dk.sds.nsp.security.SecurityContext.Organisation.OrganisationIdentifierFormat;
import dk.sundhedsdatastyrelsen.livstestamente.domain.RequestContextVO;

@Service
public class MinLogDataWrapperImpl implements MinLogDataWrapper {

    private MinLogClient minLogClient;

    public MinLogDataWrapperImpl(MinLogClient minLogClient) {
        this.minLogClient = minLogClient;
    }

    @Override
    public void logActivity(RequestContextVO context, String activity, String citizenCpr, String orgId,
                                String orgName, OrganisationIdentifierFormat orgType,
                                String userCpr, String onBehalfOfCpr, String sourceSystemIdentifier, String userRole) throws MinLogException {

        MinLogData logEntry;
        try {
            Instant dateTime = Instant.now().truncatedTo(ChronoUnit.SECONDS);
            String sessionId = context != null ? context.getMessageId() : null;
            logEntry = new MinLogData(activity, dateTime, orgId,
                    orgName, orgType, citizenCpr, sessionId, userCpr, onBehalfOfCpr,
                    sourceSystemIdentifier, userRole);
        } catch (Exception e) {
            throw new MinLogException("Error preparing arguments for MinLog registration.", e);
        }

        minLogClient.logDataAdd(context, logEntry);
    }
}
