/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.minlog.util;

import dk.sdsd.nsp.slalog.api.SLALogConfig;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sundhedsdatastyrelsen.livstestamente.domain.RequestContextVO;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

/**
 * This SlaLogger is used for SLA logging of calls to SCES and Minlog
 */
public class SlaLogger {

    private static final Logger logger = LogManager.getLogger(SlaLogger.class);

    private static final Integer lock = 1;
    private static final String SHORT_APP_NAME = "btr";
    private static SLALogConfig slaLogInstance;

    private String name;

    public SlaLogger(String name) {
        this.name = name;
        synchronized (lock) {
            if (slaLogInstance == null) {
                slaLogInstance = new SLALogConfig(SHORT_APP_NAME, SHORT_APP_NAME);
            }
        }
    }

    public void addSlaLog(RequestContextVO context, long startNanos, long startMillis, String errorMessage) {
        SLALogItem mainItem = context.getSlaLogItem();

        if (slaLogInstance != null) {
            try {
                SLALogItem logItem = slaLogInstance.getSLALogger().createLogItem(SHORT_APP_NAME + "." + name, SHORT_APP_NAME + "." + name);

                logItem.setClientIP(mainItem.getClientIP());
                logItem.setSourceSOAPAction(mainItem.getSourceSOAPAction());
                logItem.setSourceOperation(mainItem.getSourceOperation());
                logItem.setSourceEndpoint(mainItem.getSourceEndpoint());
                logItem.setMessageId(mainItem.getMessageId());
                logItem.setStartTime(startNanos, startMillis);
                logItem.setEndTime(System.nanoTime(), System.currentTimeMillis());

                if (errorMessage == null) {
                    logItem.setCallResultOk();
                } else {
                    logItem.setCallResultError(errorMessage);
                }

                logItem.store();
            } catch (Exception e) {
                logger.error("Could not create SLA LogItem", e);
            }
        }
    }
}
