/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade;

import dk.nsi.advis.v10.NotifyContent;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dto.NotifyInput;
import org.oasis_open.docs.wsn.b_2.NotificationMessageHolderType;
import org.oasis_open.docs.wsn.b_2.Notify;
import org.oasis_open.docs.wsn.b_2.TopicExpressionType;

public class NasDataMapper {
    private static final String DIALECT_SIMPLE = "http://docs.oasis-open.org/wsn/t-1/TopicExpression/Simple";
    private static final String ID_TYPE_CPR = "http://nsi.dk/advis/v10/CPR";

    public Notify toNotify(NotifyInput notifyInput) {
        Notify notify = new Notify();

        // Create topic expression
        TopicExpressionType topicExpression = new TopicExpressionType();
        topicExpression.setDialect(DIALECT_SIMPLE);
        topicExpression.getContent().add(notifyInput.getTopic());

        // Create contents
        NotifyContent nc = new NotifyContent();
        nc.setId(notifyInput.getPersonIdentifier());
        nc.setIdType(ID_TYPE_CPR);

        nc.setAny(notifyInput.getMessage());

        // Create message
        NotificationMessageHolderType message = new NotificationMessageHolderType();
        message.setTopic(topicExpression);
        message.setMessage(new NotificationMessageHolderType.Message());
        message.getMessage().setAny(nc);

        // Create request
        notify.getNotificationMessage().add(message);

        return notify;
    }
}
