/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade;

import dk.sdsd.nsp.slalog.api.SLALogConfig;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dgws.DgwsHeaderFactory;

public class NasFacadeBuilder {
    private int maxProxyPoolSize = 10;

    private NasProxy nasProxy;
    private SLALogConfig slaLogConfig;
    private int failThreshold;
    private int connectTimeoutMillis;
    private int readTimeoutMillis;
    private int maxTotalConnections;
    private int defaultMaxConnectionsPerRoute;
    private String endpoint;
    private Class<?> messageContentsType;

    private DgwsHeaderFactory dgwsHeaderFactory = null;

    public NasFacadeBuilder(SLALogConfig slaLogConfig, int failThreshold, int connectTimeoutMillis, int readTimeoutMillis, int maxTotalConnections, int defaultMaxConnectionsPerRoute, String endpoint, Class<?> messageContentsType) {
        this.slaLogConfig = slaLogConfig;
        this.failThreshold = failThreshold;
        this.connectTimeoutMillis = connectTimeoutMillis;
        this.readTimeoutMillis = readTimeoutMillis;
        this.maxTotalConnections = maxTotalConnections;
        this.defaultMaxConnectionsPerRoute = defaultMaxConnectionsPerRoute;
        this.endpoint = endpoint;
        this.messageContentsType = messageContentsType;
    }

    public NasFacadeBuilder withNasProxy(NasProxy nasProxy) {
        this.nasProxy = nasProxy;

        return this;
    }

    public NasFacadeBuilder withDgwsHeaderFactory(DgwsHeaderFactory dgwsHeaderFactory) {
        this.dgwsHeaderFactory = dgwsHeaderFactory;

        return this;
    }

    public NasFacadeBuilder setMaxProxyPoolSize(int size) {
        this.maxProxyPoolSize = size;

        return this;
    }

    public NasFacade build() {
        if(nasProxy == null) {
            nasProxy = new NasProxyImpl(endpoint, messageContentsType, connectTimeoutMillis, readTimeoutMillis, maxTotalConnections, defaultMaxConnectionsPerRoute, maxProxyPoolSize);
        }

        return new NasFacadeImpl(dgwsHeaderFactory, slaLogConfig, failThreshold, nasProxy);
    }
}
