/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade;

import org.apache.commons.pool.PoolableObjectFactory;

import javax.xml.bind.JAXBException;

public class NotificationBrokerClientObjectFactory implements PoolableObjectFactory<NotificationBrokerClient> {
    private String serviceEndpoint;
    private Class<?> messageContentsType;
    private int connectTimeoutMillis;
    private int readTimeoutMillis;
    private int maxTotalConnections;
    private int defaultMaxConnectionsPerRoute;

    NotificationBrokerClientObjectFactory(String serviceEndpoint, Class<?> messageContentsType, int connectTimeoutMillis, int readTimeoutMillis, int maxTotalConnections, int defaultMaxConnectionsPerRoute) {
        this.serviceEndpoint = serviceEndpoint;
        this.messageContentsType = messageContentsType;
        this.connectTimeoutMillis = connectTimeoutMillis;
        this.readTimeoutMillis = readTimeoutMillis;
        this.maxTotalConnections = maxTotalConnections;
        this.defaultMaxConnectionsPerRoute = defaultMaxConnectionsPerRoute;
    }

    @Override
    public NotificationBrokerClient makeObject() {
        try {
            return new NotificationBrokerClient(serviceEndpoint, messageContentsType, connectTimeoutMillis, readTimeoutMillis, maxTotalConnections, defaultMaxConnectionsPerRoute);
        } catch (JAXBException e) {
            throw new IllegalStateException("Unable to create client.", e);
        }
    }

    @Override
    public void destroyObject(NotificationBrokerClient obj) {
        // Empty
    }

    @Override
    public boolean validateObject(NotificationBrokerClient obj) {
        return true;
    }

    @Override
    public void activateObject(NotificationBrokerClient obj) {
        // Empty
    }

    @Override
    public void passivateObject(NotificationBrokerClient obj) {
        // Empty
    }
}
