/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.common;

import dk.sdsd.nsp.slalog.api.SLALogConfig;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dto.SlaLogInput;

public class SLALogFacadeImpl implements SLALogFacade {

	private SLALogConfig slaLogConfig;

	private SLALogItem slaLogItem;

	public SLALogFacadeImpl(SLALogConfig slaLogConfig) {
		this.slaLogConfig = slaLogConfig;
	}

	@Override
	public void startSLALog(String operation, SlaLogInput slaLogInput) {
		slaLogItem = createLogItem(operation, slaLogInput);
	}

	@Override
	public void completeSLALog() {
		slaLogItem.setCallResultOk();
		slaLogItem.store();
	}

	@Override
	public void completeSLALogFailure(Exception e) {
		slaLogItem.setCallResultError(e.getMessage());
		slaLogItem.store();
	}

	private SLALogItem createLogItem(String operation, SlaLogInput slaLogInput) {
		String logPoint = "MSBUtil" + "." + operation;
		SLALogItem logItem = null;
		logItem = slaLogConfig.getSLALogger().createLogItem(logPoint, slaLogInput.getShortAppName() + "." + operation);
		//			logItem.setClientIP(servletRequest);
		logItem.setSourceSOAPAction(operation);
		logItem.setSourceOperation(operation);
		//			logItem.setSourceEndpoint(servletRequest.getRequestURL().toString());
		logItem.setMessageId(slaLogInput.getMessageId());

		return logItem;
	}
}
