/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dgws;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.xpath.*;
import java.io.StringWriter;

public enum HeaderConverter {
    HSUID("//soap:Header/hsuid:HsuidHeader"),
    SECURITY("//soap:Header/wssec:Security"),
    MEDCOM("//soap:Header/medcom:Header");

    private String query;
    private static final XPath xpath = XPathFactory.newInstance().newXPath();

    private HeaderConverter(String query) {
        this.query = query;
    }

    private Node extract(Node node) {
        xpath.setNamespaceContext(new DGWSNSContext());

        try {
            XPathExpression expr = xpath.compile(this.query);
            NodeList nodeList = (NodeList)expr.evaluate(node, XPathConstants.NODESET);
            if (nodeList.getLength() > 1) {
                throw new IllegalArgumentException("Document contained more than one " + this.toString() + " header");
            } else {
                return nodeList.item(0);
            }
        } catch (XPathExpressionException var4) {
            return null;
        }
    }

    private <T> T toJaxb(Class<T> jaxbClass, Node node) {
        JAXBContext context = null;

        try {
            context = JAXBContext.newInstance(new Class[]{jaxbClass});
            JAXBElement<T> element = context.createUnmarshaller().unmarshal(node, jaxbClass);
            return element.getValue();
        } catch (Exception var5) {
            throw new HeaderConverterException(var5);
        }
    }

    public <T> T documentToJaxb(Class<T> jaxbClass, Node node) {
        Node n = this.extract(node);
        if (n == null) {
            throw new HeaderConverterException("Header " + this + " not found in document");
        } else {
            return this.toJaxb(jaxbClass, n);
        }
    }

    public String toString(Class jaxbClass, Object jaxbObject) {
        JAXBContext context = null;

        try {
            context = JAXBContext.newInstance(new Class[]{jaxbClass});
            StringWriter writer = new StringWriter();
            context.createMarshaller().marshal(jaxbObject, writer);
            return writer.toString();
        } catch (Exception var5) {
            throw new HeaderConverterException(var5);
        }
    }
}