/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dgws;


import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import java.util.HashMap;
import java.util.Map;

public class NasRequestContext {
    private static final ThreadLocal<NasRequestContext> currentRequestContext = new ThreadLocal<NasRequestContext>();

    private SLALogItem slaLogItem;
    private Security securityHeader;
    private Header medcomHeader;
    private Map<String,String> auditLogParameters;
    private String flowId;

    public static NasRequestContext get() {
        NasRequestContext requestContext = currentRequestContext.get();
        if (requestContext == null) {
            requestContext = new NasRequestContext();
            currentRequestContext.set(requestContext);
        }
        return requestContext;
    }

    public SLALogItem getSlaLogItem() {
        return slaLogItem;
    }

    public void setSlaLogItem(SLALogItem slaLogItem) {
        this.slaLogItem = slaLogItem;
    }

    public Security getSecurityHeader() {
        return securityHeader;
    }

    public void setSecurityHeader(Security securityHeader) {
        this.securityHeader = securityHeader;
    }

    public Header getMedcomHeader() {
        return medcomHeader;
    }

    public void setMedcomHeader(Header medcomHeader) {
        this.medcomHeader = medcomHeader;
    }

    public void addAuditLogParameter(String key, String value) {
        if (key == null) {
            return;
        }
        if (auditLogParameters == null) {
            auditLogParameters = new HashMap<>();
        }
        auditLogParameters.put(key, value);
    }

    public Map<String,String> getAuditLogParameters() {
        return auditLogParameters;
    }

    public String getFlowId() {
        return flowId;
    }

    public void setFlowId(String flowId) {
        this.flowId = flowId;
    }
}