/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dgws;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.SecurityTokenRequest;
import dk.sosi.seal.model.SecurityTokenResponse;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.w3c.dom.Document;

import java.io.IOException;

public class StsClientImpl implements StsClient {

	
	private HttpClient httpClient;
	
	private String stsIssueUrl;

	private SOSIFactory sosiFactory;

	public StsClientImpl(SOSIFactory sosiFactory, String stsIssueUrl) {
		this(sosiFactory, new HttpClient(), stsIssueUrl);
	}

	public StsClientImpl(SOSIFactory sosiFactory, HttpClient httpClient, String stsIssueUrl) {
		this.sosiFactory = sosiFactory;
		this.httpClient = httpClient;
		this.stsIssueUrl = stsIssueUrl;
	}
	
	@Override
	public SecurityTokenResponse requestToken(SecurityTokenRequest securityTokenRequest) throws StsException {
		
		PostMethod post = new PostMethod(stsIssueUrl);
		Document requestDocument = securityTokenRequest.serialize2DOMDocument();
		String requestString = XmlUtil.node2String(requestDocument, false, true);
		
		try {
			RequestEntity requestEntity = new StringRequestEntity(requestString, "text/xml", "utf-8");
			post.setRequestEntity(requestEntity);
			int status = httpClient.executeMethod(post);
			if (status != 200) {
				throw new StsException("Statuscode "+status+" returned from call to STS");
			}
			String responseString = post.getResponseBodyAsString();
			SecurityTokenResponse response = sosiFactory.deserializeSecurityTokenResponse(responseString);
			return response;
		} catch (IOException e) {
			throw new StsException(e);
		}
	}
}
