/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.treatmentwill;

import dk.sundhedsdatastyrelsen.livingwill._2022._08._01.LivingWillUpdated;
import dk.sundhedsdatastyrelsen.treatmentwill._2022._08._01.TreatmentWillUpdated;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;

@Component
public class NotificationBuilder {
    public TreatmentWillUpdated buildTreatmentWillNotification(String personIdentifier, Instant pointInTime) {
        TreatmentWillUpdated notification = new TreatmentWillUpdated();



        notification.setDate(getFormattedDate(pointInTime));

        notification.setType("http://sundhedsdatastyrelsen.dk/MessageDefinition/Treatmentwill-notification");

        notification.setVersion("1");

        return notification;
    }

    public LivingWillUpdated buildLivingWillNotification(String personIdentifier, Instant pointInTime) {
        LivingWillUpdated notification = new LivingWillUpdated();


        notification.setDate(getFormattedDate(pointInTime));

        notification.setType("http://sundhedsdatastyrelsen.dk/MessageDefinition/Livingwill-notification");

        notification.setVersion("1");

        return notification;
    }
    private String getFormattedDate(Instant pointInTime) {
        return DateTimeFormatter.ofPattern("yyyy-MM-dd").withZone(ZoneId.systemDefault()).format(pointInTime);
    }
}
