/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.treatmentwill;

import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.NasFacade;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dto.NotifyInput;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.facade.dto.SlaLogInput;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.Instant;

@Service
public class NotificationSender {
    @Autowired
    private NotificationBuilder notificationBuilder;

    @Autowired
    @Qualifier("TreatmentWillNasFacade")
    private NasFacade nasTreatmentWillFacade;

    @Autowired
    @Qualifier("LivingWillNasFacade")
    private NasFacade nasLivingWillFacade;

    @Value("${nas.topic.livingwillnotification}")
    private String nasTopicLivingwillnotification;


    @Value("${nas.topic.treatmentwillnotification}")
    private String nasTopicTreatmentwillnotification;

    public void sendTreatmentWillNotification(String personIdentifier, Instant pointInTime) throws NasException {
        // Create the payload
        NotifyInput notifyInput = new NotifyInput();

        notifyInput.setPersonIdentifier(personIdentifier);
        notifyInput.setTopic(nasTopicTreatmentwillnotification);
        notifyInput.setMessage(notificationBuilder.buildTreatmentWillNotification(personIdentifier, pointInTime));
        notifyInput.setSlaLogInput(getSlaLogInput());

        // Send it
        try {
            nasTreatmentWillFacade.notify(notifyInput);
        }
        catch(Exception f) {
            throw new NasException("Error invoking NAS", f);
        }
    }

    public void sendLivingWillNotification(String personIdentifier, Instant pointInTime) throws NasException {
        // Create the payload
        NotifyInput notifyInput = new NotifyInput();

        notifyInput.setPersonIdentifier(personIdentifier);
        notifyInput.setTopic(nasTopicLivingwillnotification);
        notifyInput.setMessage(notificationBuilder.buildLivingWillNotification(personIdentifier, pointInTime));
        notifyInput.setSlaLogInput(getSlaLogInput());

        // Send it
        try {
            nasLivingWillFacade.notify(notifyInput);
        }
        catch(Exception f) {
            throw new NasException("Error invoking NAS", f);
        }
    }

    private SlaLogInput getSlaLogInput() {
        SlaLogInput input = new SlaLogInput();

        input.setAppName("behandlingstestamenteregister");
        input.setShortAppName("btr");

        return input;
    }
}
