/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.util;

import java.util.HashMap;
import java.util.Map;

public class WSDLHelper {
    /**
     * Replaces relative references to schema files with URL address
     *
     * @param xml        - original XML string
     * @param requestURL - URL used to load XML page
     * @return XML string with replaced references
     */
    public static String replaceSchemaLocations(String xml, String requestURL) {
        String currentResourcePath = requestURL;
        if (currentResourcePath.endsWith("/")) {
            currentResourcePath = currentResourcePath.substring(0, currentResourcePath.length() - 1);
        }
        int pos = currentResourcePath.lastIndexOf('/');
        if (pos > 0) {
            currentResourcePath = currentResourcePath.substring(0, pos);
            Map<String, String> locationMap = new HashMap<>();

            String quoteChar = "\"";
            int startPos = 0;
            while (startPos > -1) {
                pos = startPos;
                startPos = xml.indexOf("schemaLocation=\"", pos);
                if (startPos < 0) {
                    startPos = xml.indexOf("schemaLocation='", pos);
                    quoteChar = "'";
                }
                if (startPos > 0) {
                    int endPos = xml.indexOf(quoteChar, startPos + 16);
                    if (endPos > startPos) {
                        String schemaLocation = xml.substring(startPos + 16, endPos);

                        String newResourcePath = currentResourcePath;
                        String truncatedLocation = schemaLocation;
                        while (truncatedLocation.startsWith("../")) {
                            truncatedLocation = truncatedLocation.substring(3);
                            pos = newResourcePath.lastIndexOf('/');
                            if (pos > 0) {
                                newResourcePath = newResourcePath.substring(0, pos);
                            }
                        }
                        String newSchemaLocation = newResourcePath + "/" + truncatedLocation;
                        locationMap.put(schemaLocation, newSchemaLocation);
                        startPos = endPos;
                    } else {
                        startPos = -1;
                    }
                }
            }

            for (String schemaLocation : locationMap.keySet()) {
                xml = xml.replace(schemaLocation, locationMap.get(schemaLocation));
            }
        }

        return xml;
    }
}
