/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.webservice;

import dk.sundhedsdatastyrelsen.livstestamente.service.AutowiredHttpServlet;
import org.apache.log4j.Logger;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import static java.net.HttpURLConnection.HTTP_BAD_REQUEST;

@WebServlet(name="TreatmentWillDispatcherServlet", urlPatterns={"/btr"})
public class TreatmentWillDispatcherServlet extends AutowiredHttpServlet {

    private static final Logger LOGGER = Logger.getLogger(TreatmentWillDispatcherServlet.class);

	private final Map<String, String> dispatcherMap = new HashMap<String, String>();

	public TreatmentWillDispatcherServlet() {
		dispatcherMap.put("http://sundhedsdatastyrelsen.dk/behandlingstestamente/2018/05/01", "/btr/20180501");

		dispatcherMap.put("http://sundhedsdatastyrelsen.dk/behandlingstestamente/2020/03/16", "/btr/20200316");
	}
	
	@Override
	protected void service(HttpServletRequest req, HttpServletResponse response) throws ServletException, IOException {
		String soapAction = req.getHeader("SOAPAction");
		
		if (soapAction != null) {
			String lookup = soapAction;
			if (soapAction.contains("#")) {
				String[] parts = soapAction.split("#");
				lookup = parts[0];
				lookup = lookup.replaceAll("\"", "");
			}
			
			String dispatchTarget = dispatcherMap.get(lookup);
			
			if (dispatchTarget != null) {
				RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(dispatchTarget);
				dispatcher.forward(req, response);
				return;
			}
		}
		
		LOGGER.info("Urutbar soapaction :"+soapAction+" modtaget");
        response.getWriter().println("SOAPAction skal være en af følgende: " + dispatcherMap.keySet());
        response.setStatus(HTTP_BAD_REQUEST);
	}
}
