/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.webservice.hl7cda;

import dk.sundhedsdatastyrelsen.livstestamente.OID;
import dk.sundhedsdatastyrelsen.livstestamente.domain.LivingWillVO;
import dk.sundhedsdatastyrelsen.livstestamente.domain.RequestContextVO;
import org.hl7.ltr.LivingWill;
import org.hl7.v3.POCDMT000040ClinicalDocument;
import org.springframework.stereotype.Component;

@Component
public class LivingWillHL7CDAMapper extends AbstractHL7CDAMapper {

    public static org.hl7.ltr.ObjectFactory oLTR = new org.hl7.ltr.ObjectFactory();

    private static final String LIVING_WILL_CODE = "LivingWill";
    private static final String LIVING_WILL_DISPLAY_NAME = "Livstestamente";

    public POCDMT000040ClinicalDocument toClinicalDocument(LivingWillVO livingWillVO, RequestContextVO requestContextVO) {
        final POCDMT000040ClinicalDocument doc = createDocument(LIVING_WILL_CODE, LIVING_WILL_DISPLAY_NAME, OID.LIVING_WILL_CODE_SYSTEM);

        // effectiveTime
        doc.setEffectiveTime(createTimestamp(requestContextVO.getTime()));

        // versionNumber
        doc.setVersionNumber(createVersionNumber(livingWillVO.getVersion()));

        // recordTarget
        doc.getRecordTargets().add(createRecordTarget(livingWillVO.getPersonIdentifier()));

        // author
        doc.getAuthors().add(createAuthor(livingWillVO.getPersonIdentifier(), livingWillVO.getModifiedDateTime()));

        // custodian
        doc.setCustodian(createCustodian());

        // component
        LivingWill livingWill = mapLivingWillValues(livingWillVO);
        doc.setComponent(createComponent(LIVING_WILL_DISPLAY_NAME, LIVING_WILL_CODE, OID.LIVING_WILL_CODE_SYSTEM, livingWill));

        return doc;
    }

    public LivingWillVO createLivingWillFromRequest(String personIdentifier, LivingWill livingWill) {
        // Notice: It is assumed that the structure has been checked by the validator.

        boolean noLifeProlongingIfDying = livingWill.isNoLifeProlongingIfDying();
        boolean noLifeProlongingIfSeverelyDegraded = livingWill.isNoLifeProlongingIfSeverelyDegraded();

        LivingWillVO livingWillVO = new LivingWillVO();
        livingWillVO.setPersonIdentifier(personIdentifier);
        livingWillVO.setNoLifeProlongingIfDying(noLifeProlongingIfDying);
        livingWillVO.setNoLifeProlongingIfSeverelyDegraded(noLifeProlongingIfSeverelyDegraded);
        return livingWillVO;
    }

    private LivingWill mapLivingWillValues(LivingWillVO livingWillVO) {
        LivingWill livingWill = new LivingWill();
        livingWill.setNoLifeProlongingIfDying(livingWillVO.isNoLifeProlongingIfDying());
        livingWill.setNoLifeProlongingIfSeverelyDegraded(livingWillVO.isNoLifeProlongingIfSeverelyDegraded());
        return livingWill;
    }
}
