/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.business.validator;

import dk.sundhedsdatastyrelsen.livstestamente.domain.TreatmentWillVO;
import dk.sundhedsdatastyrelsen.livstestamente.util.FaultCode;
import dk.sundhedsdatastyrelsen.livstestamente.util.ServiceException;
import dk.sundhedsdatastyrelsen.livstestamente.service.util.TestHelperV2;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

public class TreatmentWillValidatorTests extends TestHelperV2 {

    private TreatmentWillValidator classUnderTest;

    @Before
    public void setUp() {
        classUnderTest = new TreatmentWillValidator();
    }

    @Test
    public void validateTreatmentWillExists_fail() {
        try {
            classUnderTest.validateTreatmentWillExists(null);
            fail();
        } catch (ServiceException e) {
            assertEquals(FaultCode.TREATMENT_WILL_NO_DATA_FOUND, e.getFaultCode());
        }
    }

    @Test
    public void validateTreatmentWillNotExists_fail() {
        TreatmentWillVO treatmentWillVO = new TreatmentWillVO();
        try {
            classUnderTest.validateTreatmentWillNotExists(treatmentWillVO);
            fail();
        } catch (ServiceException e) {
            assertEquals(FaultCode.TREATMENT_WILL_ALREADY_EXISTING, e.getFaultCode());
        }
    }
}
