/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.conf;

import dk.nsp.test.idp.OrganizationIdentities;
import dk.sosi.seal.vault.CredentialVault;
import dk.sundhedsdatastyrelsen.livstestamente.conf.Properties;
import org.mockito.Mockito;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;

import dk.sds.nsp.security.DevelopmentSecurityProvider;
import dk.sds.nsp.security.SecurityContext;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.minlog.MinLogClient;
import dk.sundhedsdatastyrelsen.livstestamente.service.context.RequestContextProvider;
import dk.sundhedsdatastyrelsen.livstestamente.service.context.SecurityContextProvider;
import dk.sundhedsdatastyrelsen.livstestamente.context.ServiceActorProvider;

@Configuration
@Profile("development")
@ComponentScan("dk.sundhedsdatastyrelsen.livstestamente")
public class TestConfig {

    // In normal test cases we always mock MinLogClient to avoid calling an external service
    @Bean
    public MinLogClient minLogClient() {
        return Mockito.mock(MinLogClient.class);
    }
    
    @Bean
    public SecurityContextProvider securityContextProvider() {
    	return new SecurityContextProvider() {
    		@Override
    		public SecurityContext getSecurityContext() {
    			return DevelopmentSecurityProvider.provider().getSecurityContext();
    		}
    	};
    }
    
    @Bean
    public RequestContextProvider requestContextProvider(SecurityContextProvider securityContextProvider, ServiceActorProvider serviceActorProvider) {
    	return new RequestContextProvider(securityContextProvider, serviceActorProvider);
    }

    @Bean
    public ServiceActorProvider serviceActorProvider() {
    	return Mockito.mock(ServiceActorProvider.class);
    }

    @Bean
    public CredentialVaultFactory credentialVaultFactory() {
        return new CredentialVaultFactory() {
            @Override
            public CredentialVault getCredentilVaultFromProperties(Properties properties, java.util.Properties sosiProperties) {
                return OrganizationIdentities.sundhedsdatastyrelsen().getServiceConsumer().getCredentialVault();
            }
        };
    }
}
