/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.conf;

import org.flywaydb.core.Flyway;
import org.junit.ClassRule;
import org.junit.rules.TemporaryFolder;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabase;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseBuilder;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseType;
import org.springframework.transaction.PlatformTransactionManager;

import javax.sql.DataSource;
import java.io.File;
import java.io.IOException;

@Configuration
@Profile("development")
public class TestDAOConfig {

    @ClassRule
    private TemporaryFolder tempDBFolder = new TemporaryFolder();
    TestDBScriptPreprocessor scriptPreprocessor = new TestDBScriptPreprocessor();

    private final Flyway flywayTreatmentWill = new Flyway();

    @Bean
    public DataSource btrDataSource() {
        DataSource datasource = createEmbeddedDatabase("btr");
        File scriptsLocation = prepareScripts("../compose/database/db/migration/btr", "../compose/database/db/test/btr");

        flywayTreatmentWill.setDataSource(datasource);
        flywayTreatmentWill.setLocations("filesystem:/" + scriptsLocation.getPath());
        flywayTreatmentWill.migrate();
        return datasource;
    }

    @Bean
    public PlatformTransactionManager btrDataSourceTxManager(@Qualifier("btrDataSource") DataSource btrDataSource) {
        return new DataSourceTransactionManager(btrDataSource);
    }

    private File prepareScripts(String... paths) {
        try {
            tempDBFolder.create();
            File subFolder = tempDBFolder.newFolder();
            for (String path : paths) {
                scriptPreprocessor.preprocessScripts(path, subFolder);
            }
            return subFolder;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    private EmbeddedDatabase createEmbeddedDatabase(String name) {
        return new EmbeddedDatabaseBuilder()
                .setType(EmbeddedDatabaseType.H2)
                .setName(name + ";DB_CLOSE_DELAY=-1;MODE=MySQL")
                .build();
    }
}
