/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.conf;

import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Stream;

public class TestDBScriptPreprocessor {

    public void preprocessScripts(String sourcePath, File destFolder) {

        String[] scriptPaths = getScripts(sourcePath);
        Map<String, String> sqlScripts = new HashMap<>();

        for (String path : scriptPaths) {
            try {
                StringBuilder sql = new StringBuilder();

                for (String line : Files.readAllLines(new File(path).toPath())) {
                    // The H2 database was added after the SQL scripts were written, and we would like not to change the
                    // scripts. Preprocess the SQL scripts to ignore collate-directives (not supported by H2) to avoid a casting error.

                    line = line
                            .replace("USE ltr;", "")
                            .replace("USE btr;", "")
                            .replace("USE stm_btr;", "")
                            .replace("COLLATE=utf8_bin", "")
                            .replace("COLLATE 'utf8_bin'", "")
                            .replace("COLLATE = utf8_bin", "")
                            .replace("TINYINT", "INT")
                            .replaceAll("(?i)(COMMENT ).*([,;])", "$2")
                            .replace("\"", "'");
                    sql.append(line + "\n");
                }
                String fileName = path.substring(path.lastIndexOf("/")+1);
                sqlScripts.put(fileName, sql.toString());
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        for (String sqlScript: sqlScripts.keySet()) {
            try {
                final File tempFile = new File(destFolder, sqlScript);
                FileUtils.writeStringToFile(tempFile, sqlScripts.get(sqlScript));

            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }


    private String[] getScripts(String sourcePath) {
        File dir = new File(sourcePath);

        try (Stream<Path> fs = Files.list(dir.toPath())) {
            return fs.filter(Files::isRegularFile)
                    .map(Path::toString)
                    .sorted()
                    .toArray(String[]::new);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
