/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.conf;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.ws.SLALoggingServletFilter;
import dk.sundhedsdatastyrelsen.livstestamente.service.AdminServlet;
import dk.sundhedsdatastyrelsen.livstestamente.service.webservice.LivingWillWebService;
import dk.sundhedsdatastyrelsen.livstestamente.service.webservice.LtrBtrContextListener;
import dk.sundhedsdatastyrelsen.livstestamente.service.webservice.TreatmentWillWebService;
import dk.sundhedsdatastyrelsen.livstestamente.service.webservice.TreatmentWillWebServiceV2;
import dk.sundhedsdatastyrelsen.livstestamente.integrations.personInformation.PersonInformation;
import dk.sundhedsdatastyrelsen.livstestamente.integrations.personInformation.impl.PersonInformationServiceClient;
import dk.sundhedsdatastyrelsen.livstestamente.integrations.personInformation.impl.PersonInformationServiceClientFactory;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.context.support.GenericApplicationContext;

import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

@Configuration
@Profile("development")
public class TestWebServiceConfig {

    @Bean
    public LivingWillWebService livingWillWebService(GenericApplicationContext springContext) {
        LivingWillWebService livingWillWebService = new LivingWillWebService();
        livingWillWebService.setContext(createWebServiceContext(springContext, false));
        return livingWillWebService;
    }

    @Bean
    public LivingWillWebService livingWillAdminWebService(GenericApplicationContext springContext,
                                                          @Autowired PersonInformation personInformation) {
        LivingWillWebService livingWillWebService = new LivingWillWebService();
        personInformation.setServiceClient(createFakeServiceClient());
        livingWillWebService.setContext(createWebServiceContext(springContext, true));
        return livingWillWebService;
    }

    private PersonInformationServiceClient createFakeServiceClient() {
        int age = 18;
        PersonInformationServiceClient personinformationServiceClient = Mockito.mock(PersonInformationServiceClient.class);
        Mockito.when(personinformationServiceClient.getAge(Mockito.anyString())).thenReturn(age + 1);

        PersonInformationServiceClientFactory personInformationServiceClientFactory = Mockito.mock(PersonInformationServiceClientFactory.class);
        Mockito.when(personInformationServiceClientFactory.createClient()).thenReturn(personinformationServiceClient);
        return personInformationServiceClientFactory.createClient();
    }

    @Bean
    public TreatmentWillWebService treatmentWillWebService(GenericApplicationContext springContext) {
        TreatmentWillWebService treatmentWillWebService = new TreatmentWillWebService();
        treatmentWillWebService.setContext(createWebServiceContext(springContext, false));
        return treatmentWillWebService;
    }

    @Bean
    public TreatmentWillWebService treatmentWillAdminWebService(GenericApplicationContext springContext) {
        TreatmentWillWebService treatmentWillWebService = new TreatmentWillWebService();
        treatmentWillWebService.setContext(createWebServiceContext(springContext, true));
        return treatmentWillWebService;
    }

    @Bean
    public TreatmentWillWebServiceV2 treatmentWillWebServiceV2(GenericApplicationContext springContext) {
        TreatmentWillWebServiceV2 treatmentWillWebServiceV2 = new TreatmentWillWebServiceV2();
        treatmentWillWebServiceV2.setContext(createWebServiceContext(springContext, false));
        return treatmentWillWebServiceV2;
    }

    @Bean
    public TreatmentWillWebServiceV2 treatmentWillAdminWebServiceV2(GenericApplicationContext springContext) {
        TreatmentWillWebServiceV2 treatmentWillWebServiceV2 = new TreatmentWillWebServiceV2();
        treatmentWillWebServiceV2.setContext(createWebServiceContext(springContext, true));
        return treatmentWillWebServiceV2;
    }

    private WebServiceContext createWebServiceContext(GenericApplicationContext springContext, boolean isAdmin) {
        SLALogItem slaLogItem = Mockito.mock(SLALogItem.class);

        ServletContext servletContext = Mockito.mock(ServletContext.class);
        Mockito.when(servletContext.getAttribute(Mockito.eq(LtrBtrContextListener.ANNOTATION_CONFIG_CONTEXT))).thenReturn(springContext);
        ServletRequest servletRequest = Mockito.mock(ServletRequest.class);
        Mockito.when(servletRequest.getAttribute(Mockito.eq(SLALoggingServletFilter.CURRENT_LOGITEM_REQUEST_KEY))).thenReturn(slaLogItem);
        Mockito.when(servletRequest.getAttribute(Mockito.eq(AdminServlet.ADMIN_KEY))).thenReturn(isAdmin);

        MessageContext messageContext = Mockito.mock(MessageContext.class);
        Mockito.when(messageContext.get(Mockito.eq(MessageContext.SERVLET_CONTEXT))).thenReturn(servletContext);
        Mockito.when(messageContext.get(Mockito.eq(MessageContext.SERVLET_REQUEST))).thenReturn(servletRequest);

        WebServiceContext webServiceContext = Mockito.mock(WebServiceContext.class);
        Mockito.when(webServiceContext.getMessageContext()).thenReturn(messageContext);
        return webServiceContext;
    }
}
