/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.minlog;

import dk.sds.nsp.security.SecurityContext.Organisation.OrganisationIdentifierFormat;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.ArgumentMatchers;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;

import java.time.Instant;
import java.time.temporal.ChronoUnit;

public class MinLogDataWrapperTests {

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Mock
    private MinLogClient minLogClient;

    private MinLogDataWrapperImpl classUnderTest;

    @Before
    public void setUp() {
        classUnderTest = new MinLogDataWrapperImpl(minLogClient);
    }

    @Test
    public void testMinLog_WithValidData_PassesDataCorrectlyToMinLogDataWrapper() throws MinLogException {
        classUnderTest.logActivity(
                null,
                "test",
                "cpr1",
                "orgId",
                "orgName",
                OrganisationIdentifierFormat.CVR,
                "cpr2",
                "cpr3",
                "system",
                "borger"
        );

        ArgumentCaptor<MinLogData> argument = ArgumentCaptor.forClass(MinLogData.class);
        Mockito.verify(minLogClient).logDataAdd(ArgumentMatchers.any(), argument.capture());
        final MinLogData minLogData = argument.getValue();
        Assert.assertFalse(minLogData.getActivity().isEmpty());
        Assert.assertEquals("test", minLogData.getActivity());
        Assert.assertEquals("cpr1", minLogData.getCitizenCpr());
        Assert.assertEquals("orgName", minLogData.getOrganizationName());
        Assert.assertEquals("orgId", minLogData.getOrganizationId());
        Assert.assertEquals(OrganisationIdentifierFormat.CVR, minLogData.getOrganizationType());
        Assert.assertEquals("cpr3", minLogData.getOnBehalfOfCpr());
        Assert.assertNull(minLogData.getSessionId());
        Assert.assertEquals("system", minLogData.getSourceSystemIdentifier());
        Instant tenMinutesAgo = Instant.now().minusSeconds(60*10).truncatedTo(ChronoUnit.SECONDS);
        Assert.assertTrue(minLogData.getEventDateTime().isAfter(tenMinutesAgo));
        Assert.assertEquals("cpr2", minLogData.getUserCPR());
    }

}
