/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.minlog;

import dk.sundhedsdatastyrelsen.livstestamente.context.ServiceActor;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;

import dk.sds.nsp.security.SecurityContext.Organisation.OrganisationIdentifierFormat;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sundhedsdatastyrelsen.livstestamente.service.AbstractSpringTest;
import dk.sundhedsdatastyrelsen.livstestamente.domain.RequestContextVO;
import dk.sundhedsdatastyrelsen.livstestamente.domain.UserType;
import dk.sundhedsdatastyrelsen.livstestamente.conf.Properties;

public class MinLogHoleThroughTest extends AbstractSpringTest {

    // Warning: Creates data in the test environment for every test run. See minlogclient.properties to know where.

    @Autowired
    private Properties properties;

    @Autowired
    private MinLogDataWrapper minLogDataWrapper;

    @Test
    public void testActualLoggingToMinLogServiceLTR() throws Exception {
        ServiceActor serviceActor = new ServiceActor(UserType.HealthCareProfessional);
		RequestContextVO requestContextMock = new RequestContextVO(serviceActor, Mockito.mock(SLALogItem.class));

        minLogDataWrapper.logActivity(
                requestContextMock,
                properties.getMinLogReadActivityTextLTR(),
                "0501792275",
                "12345678901234567890",
                "Regionshospital Randers, Akutcenter",
                OrganisationIdentifierFormat.CVR,
                "0501792275",
                "2512489996",
                "LTR integration test",
                "sundhedsfaglig"
        );
    }

    @Test
    public void testActualLoggingToMinLogServiceBTR() throws Exception {
        ServiceActor serviceActor = new ServiceActor(UserType.HealthCareProfessional);
		RequestContextVO requestContextMock = new RequestContextVO(serviceActor, Mockito.mock(SLALogItem.class));

        minLogDataWrapper.logActivity(
                requestContextMock,
                properties.getMinLogReadActivityTextBTR(),
                "0501792275",
                "12345678901234567890",
                "Regionshospital Randers, Akutcenter",
                OrganisationIdentifierFormat.CVR,
                "0501792275",
                "2512489996",
                "BTR integration test",
                "sundhedsfaglig"
        );
    }

}
