/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.integrations.treatmentwill;

import dk.sundhedsdatastyrelsen.livingwill._2022._08._01.LivingWillUpdated;
import dk.sundhedsdatastyrelsen.livstestamente.service.integrations.nas.treatmentwill.NotificationBuilder;
import dk.sundhedsdatastyrelsen.treatmentwill._2022._08._01.TreatmentWillUpdated;
import org.junit.Test;

import java.time.Instant;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

public class NotificationBuilderTest {
    private NotificationBuilder subject = new NotificationBuilder();


    @Test
    public void buildTreatmentWillNotification_populatesType() {
        // Arrange
        String personIdentifier = "1212121212";
        Instant pointInTime = Instant.parse("2021-02-12T10:15:30.00Z");

        // Act
        TreatmentWillUpdated notification = subject.buildTreatmentWillNotification(personIdentifier, pointInTime);

        // Assert
        assertEquals("http://sundhedsdatastyrelsen.dk/MessageDefinition/Treatmentwill-notification", notification.getType());
    }

    @Test
    public void buildTreatmentWillNotification_populatesVersion() {
        // Arrange
        String personIdentifier = "1212121212";
        Instant pointInTime = Instant.parse("2021-02-12T10:15:30.00Z");

        // Act
        TreatmentWillUpdated notification = subject.buildTreatmentWillNotification(personIdentifier, pointInTime);

        // Assert
        assertEquals("1", notification.getVersion());
    }

    @Test
    public void buildLivingWillNotification_populatesId() {
        // Arrange
        String personIdentifier = "1212121212";
        Instant pointInTime = Instant.parse("2021-02-12T10:15:30.00Z");

        // Act
        LivingWillUpdated notification = subject.buildLivingWillNotification(personIdentifier, pointInTime);

        // Assert
        assertNotNull(notification);
    }

    @Test
    public void buildLivingWillNotification_populatesDate() {
        // Arrange
        String personIdentifier = "1212121212";
        Instant pointInTime = Instant.parse("2021-02-12T10:15:30.00Z");

        // Act
        LivingWillUpdated notification = subject.buildLivingWillNotification(personIdentifier, pointInTime);

        // Assert
        assertEquals("2021-02-12", notification.getDate());
    }

    @Test
    public void buildLivingWillNotification_populatesType() {
        // Arrange
        String personIdentifier = "1212121212";
        Instant pointInTime = Instant.parse("2021-02-12T10:15:30.00Z");

        // Act
        LivingWillUpdated notification = subject.buildLivingWillNotification(personIdentifier, pointInTime);

        // Assert
        assertEquals("http://sundhedsdatastyrelsen.dk/MessageDefinition/Livingwill-notification", notification.getType());
    }

    @Test
    public void buildLivingWillNotification_populatesVersion() {
        // Arrange
        String personIdentifier = "1212121212";
        Instant pointInTime = Instant.parse("2021-02-12T10:15:30.00Z");

        // Act
        LivingWillUpdated notification = subject.buildLivingWillNotification(personIdentifier, pointInTime);

        // Assert
        assertEquals("1", notification.getVersion());
    }

}