/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.util;

import java.time.Instant;
import java.util.List;

import dk.sundhedsdatastyrelsen.behandlingstestamente._2020._03._16.*;
import dk.sundhedsdatastyrelsen.livstestamente.domain.AcceptanceNeeded;
import dk.sundhedsdatastyrelsen.livstestamente.domain.LivingWillVO;
import dk.sundhedsdatastyrelsen.livstestamente.domain.RequestContextVO;
import dk.sundhedsdatastyrelsen.livstestamente.domain.TreatmentWillVO;
import dk.sundhedsdatastyrelsen.livstestamente.domain.UserType;
import org.hl7.btrv2.TreatmentWillV2;
import org.hl7.v3.POCDMT000040ClinicalDocument;
import org.hl7.v3.POCDMT000040Entry;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sundhedsdatastyrelsen.livstestamente.AssigningAuthorities;
import dk.sundhedsdatastyrelsen.livstestamente.OID;
import dk.sundhedsdatastyrelsen.livstestamente.service.business.LivingWillService;
import dk.sundhedsdatastyrelsen.livstestamente.service.business.TreatmentWillService;
import dk.sundhedsdatastyrelsen.livstestamente.service.webservice.hl7cda.TreatmentWillHL7CDAMapperV2;
import dk.sundhedsdatastyrelsen.livstestamente.context.ServiceActor;

@Component
public class TestHelperV2 {

    public static final String ASSIGNING_AUTHORITY_NAME_PERSON = AssigningAuthorities.CPR;
    public static final String ORG_NAME = "OrgName";
    public static final String ORG_ID = OID.SOR;

    public static final String PERSON_IDENTIFIER = "0501792276";

    public static final String PERSON_IDENTIFIER_2 = "0501792277";

    public static final String PERSON_IDENTIFIER_3 = "0501792278";

    public static final String PERSON_IDENTIFIER_4 = "0501792279";

    public static final String ADMIN_PERSON_IDENTIFIER = "18374828181";

    public static final String MESSAGE_ID = "123";

    public static final boolean NO_LIFE_PROLONGING_IF_DYING = true;

    public static final boolean NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED = true;
    public static final AcceptanceNeeded NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED_ACCEPTANCE_NEEDED = AcceptanceNeeded.RELATIVE_ACCEPTANCE_REQUIRED;

    public static final boolean NO_LIFE_PROLONGING_IF_SEVERE_PAIN = true;
    public static final AcceptanceNeeded NO_LIFE_PROLONGING_IF_SEVERE_PAIN_ACCEPTANCE_NEEDED = AcceptanceNeeded.RELATIVE_ACCEPTANCE_REQUIRED;

    public static final boolean NO_FORCED_TREATMENT_IF_INCAPABLE = true;
    public static final AcceptanceNeeded NO_FORCED_TREATMENT_IF_INCAPABLE_ACCEPTANCE_NEEDED = AcceptanceNeeded.RELATIVE_ACCEPTANCE_REQUIRED;

    public ServiceActor getServiceActorExample() {
        ServiceActor serviceActorVO = new ServiceActor(UserType.Citizen);
        serviceActorVO.setOrganizationId(ORG_ID);
        serviceActorVO.setOrganizationName(ORG_NAME);
        serviceActorVO.setPersonIdentifier(PERSON_IDENTIFIER);
        serviceActorVO.setUserTitle("borger");
        return serviceActorVO;
    }

    public ServiceActor getServiceActorWithExistingLivingWillExample() {
        ServiceActor serviceActorVO = new ServiceActor(UserType.Citizen);
        serviceActorVO.setOrganizationId(ORG_ID);
        serviceActorVO.setOrganizationName(ORG_NAME);
        serviceActorVO.setPersonIdentifier(PERSON_IDENTIFIER_2);
        serviceActorVO.setUserTitle("borger");
        return serviceActorVO;
    }

    public ServiceActor getServiceActorAdminExample() {
        ServiceActor serviceActorVO = new ServiceActor(UserType.Admin);
        serviceActorVO.setOrganizationId(ORG_ID);
        serviceActorVO.setOrganizationName(ORG_NAME);
        serviceActorVO.setPersonIdentifier(PERSON_IDENTIFIER);
        return serviceActorVO;
    }


    public ServiceActor getServiceActorSystemUserExample() {
        ServiceActor serviceActorVO = new ServiceActor(UserType.SystemUser);
        serviceActorVO.setOrganizationId(ORG_ID);
        serviceActorVO.setOrganizationName(ORG_NAME);
        return serviceActorVO;
    }

    public ServiceActor getServiceActorHealthCareProfessionalExample() {
        ServiceActor serviceActorVO = new ServiceActor(UserType.HealthCareProfessional);
        serviceActorVO.setOrganizationId(ORG_ID);
        serviceActorVO.setOrganizationName(ORG_NAME);
        serviceActorVO.setPersonIdentifier(PERSON_IDENTIFIER);
        serviceActorVO.setUserTitle("healthcare1");
        return serviceActorVO;
    }

    public ServiceActor getServiceActorExample(UserType userType) {
        ServiceActor serviceActorVO = new ServiceActor(userType);
        serviceActorVO.setOrganizationId(ORG_ID);
        serviceActorVO.setOrganizationName(ORG_NAME);
        serviceActorVO.setPersonIdentifier(PERSON_IDENTIFIER);
        return serviceActorVO;
    }

    public RequestContextVO getRequestContextExample(ServiceActor serviceActor, String messageId) {
        RequestContextVO requestContextVO = new RequestContextVO(serviceActor, Mockito.mock(SLALogItem.class));
        requestContextVO.setTime(Instant.now());
        requestContextVO.setPersonIdentifier(PERSON_IDENTIFIER);
        requestContextVO.setMessageId(messageId);
        return requestContextVO;
    }

    public LivingWillVO getLivingWillExample() {
        LivingWillVO livingWillVO = new LivingWillVO();
        livingWillVO.setPersonIdentifier(PERSON_IDENTIFIER);
        livingWillVO.setNoLifeProlongingIfDying(NO_LIFE_PROLONGING_IF_DYING);
        livingWillVO.setNoLifeProlongingIfSeverelyDegraded(NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED);
        return livingWillVO;
    }

    public TreatmentWillVO getTreatmentWillExample() {
        TreatmentWillVO treatmentWillVO = new TreatmentWillVO();
        treatmentWillVO.setPersonIdentifier(PERSON_IDENTIFIER);
        treatmentWillVO.setNoLifeProlongingIfDying(NO_LIFE_PROLONGING_IF_DYING);

        treatmentWillVO.setNoLifeProlongingIfSeverelyDegraded(NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED);
        treatmentWillVO.setNoLifeProlongingIfSeverelyDegradedAcceptanceNeeded(NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED_ACCEPTANCE_NEEDED);
        treatmentWillVO.setNoLifeProlongingIfSeverePain(NO_LIFE_PROLONGING_IF_SEVERE_PAIN);
        treatmentWillVO.setNoLifeProlongingIfSeverePainAcceptanceNeeded(NO_LIFE_PROLONGING_IF_SEVERE_PAIN_ACCEPTANCE_NEEDED);
        treatmentWillVO.setNoForcedTreatmentIfIncapable(NO_FORCED_TREATMENT_IF_INCAPABLE);
        treatmentWillVO.setNoForcedTreatmentIfIncapableAcceptanceNeeded(NO_FORCED_TREATMENT_IF_INCAPABLE_ACCEPTANCE_NEEDED);
        return treatmentWillVO;
    }

    public LivingWillVO getLivingWill(LivingWillService service, String personIdentifier) {
        ServiceActor serviceActorExample = getServiceActorExample();
        serviceActorExample.setPersonIdentifier(personIdentifier);

        RequestContextVO context = getRequestContextExample(serviceActorExample, MESSAGE_ID);
        context.setPersonIdentifier(personIdentifier);


        return service.getLivingWill(context);
    }

    public TreatmentWillVO getTreatmentWill(TreatmentWillService service, String personIdentifier) {
        ServiceActor serviceActorExample = getServiceActorExample();
        serviceActorExample.setPersonIdentifier(personIdentifier);

        RequestContextVO context = getRequestContextExample(serviceActorExample,MESSAGE_ID);
        context.setPersonIdentifier(personIdentifier);


        return service.getTreatmentWill(context, "Logning (TEST)");
    }

    @Component
    public class HL7Helper {

        @Autowired
        public TreatmentWillHL7CDAMapperV2 treatmentWillHL7CDAMapper;

        private final dk.sundhedsdatastyrelsen.behandlingstestamente._2020._03._16.ObjectFactory factoryBTR = new dk.sundhedsdatastyrelsen.behandlingstestamente._2020._03._16.ObjectFactory();

        public List<POCDMT000040Entry> getAllEntries(POCDMT000040ClinicalDocument d) {
            return d.getComponent().getStructuredBody().getComponents().get(0).getSection().getEntries();
        }

        public TreatmentWillV2 getTreatmentWillExample() {
            final TreatmentWillV2 treatmentWill = TreatmentWillHL7CDAMapperV2.oBTR.createTreatmentWillV2();
            treatmentWill.setNoLifeProlongingIfDying(treatmentWillHL7CDAMapper.toTreatmentWillValueTypeWithoutAcceptanceNeeded(NO_LIFE_PROLONGING_IF_DYING));
            treatmentWill.setNoLifeProlongingIfSeverelyDegraded(treatmentWillHL7CDAMapper.toTreatmentWillValueType(NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED, NO_LIFE_PROLONGING_IF_SEVERELY_DEGRADED_ACCEPTANCE_NEEDED));
            treatmentWill.setNoLifeProlongingIfSeverePain(treatmentWillHL7CDAMapper.toTreatmentWillValueType(NO_LIFE_PROLONGING_IF_SEVERE_PAIN, NO_LIFE_PROLONGING_IF_SEVERE_PAIN_ACCEPTANCE_NEEDED));
            treatmentWill.setNoForcedTreatmentIfIncapable(treatmentWillHL7CDAMapper.toTreatmentWillValueType(NO_FORCED_TREATMENT_IF_INCAPABLE, NO_FORCED_TREATMENT_IF_INCAPABLE_ACCEPTANCE_NEEDED));
            return treatmentWill;
        }

        public CreateTreatmentWillRequest createCreateTreatmentWillRequest(String personIdentifier, TreatmentWillV2 treatmentWill) {
            final CreateTreatmentWillRequest request = factoryBTR.createCreateTreatmentWillRequest();
            request.setId(treatmentWillHL7CDAMapper.createId(OID.CPR, personIdentifier, TestHelperV2.ASSIGNING_AUTHORITY_NAME_PERSON));
            request.setTreatmentWillV2(treatmentWill);
            return request;
        }

        public UpgradeToTreatmentWillRequest createUpgradeToTreatmentWillRequest(String personIdentifier, TreatmentWillV2 treatmentWill) {
            final UpgradeToTreatmentWillRequest request = factoryBTR.createUpgradeToTreatmentWillRequest();
            request.setId(treatmentWillHL7CDAMapper.createId(OID.CPR, personIdentifier, TestHelperV2.ASSIGNING_AUTHORITY_NAME_PERSON));
            request.setTreatmentWillV2(treatmentWill);
            return request;
        }

        public UpdateTreatmentWillRequest createUpdateTreatmentWillRequest(String personIdentifier, TreatmentWillV2 treatmentWill) {
            final UpdateTreatmentWillRequest request = factoryBTR.createUpdateTreatmentWillRequest();
            request.setId(treatmentWillHL7CDAMapper.createId(OID.CPR, personIdentifier, TestHelperV2.ASSIGNING_AUTHORITY_NAME_PERSON));
            request.setTreatmentWill(treatmentWill);
            return request;
        }

        public DeleteTreatmentWillRequest createDeleteTreatmentWillRequest(String personIdentifier) {
            DeleteTreatmentWillRequest request = factoryBTR.createDeleteTreatmentWillRequest();
            request.setId(treatmentWillHL7CDAMapper.createId(OID.CPR, personIdentifier, TestHelperV2.ASSIGNING_AUTHORITY_NAME_PERSON));
            return request;
        }

        public GetTreatmentWillRequest createGetTreatmentWillRequest(String personIdentifier) {
            GetTreatmentWillRequest request = factoryBTR.createGetTreatmentWillRequest();
            request.setId(treatmentWillHL7CDAMapper.createId(OID.CPR, personIdentifier, TestHelperV2.ASSIGNING_AUTHORITY_NAME_PERSON));
            return request;
        }

        public GetTreatmentWillWithOnlyForcedTreatmentRequest createGetTreatmentWillWithOnlyForcedTreatmentRequest(String personIdentifier) {
            GetTreatmentWillWithOnlyForcedTreatmentRequest request = factoryBTR.createGetTreatmentWillWithOnlyForcedTreatmentRequest();
            request.setId(treatmentWillHL7CDAMapper.createId(OID.CPR, personIdentifier, TestHelperV2.ASSIGNING_AUTHORITY_NAME_PERSON));
            return request;
        }

        public HasTreatmentWillRequest createHasTreatmentWillRequest(String personIdentifier) {
            HasTreatmentWillRequest request = factoryBTR.createHasTreatmentWillRequest();
            request.setId(treatmentWillHL7CDAMapper.createId(OID.CPR, personIdentifier, TestHelperV2.ASSIGNING_AUTHORITY_NAME_PERSON));
            return request;
        }

    }

}
