/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.util;

import org.junit.Test;

import static org.junit.Assert.assertTrue;

public class WSDLHelperTest {

    @Test
    public void testReplaceLocationWithDoubleQuote() {
        String testXML = "<xs:schema ...\n" +
                "            <xs:include schemaLocation=\"../schema/wss/2004/01/testdir/test.xsd\"/>\n" +
                "          </xs:schema>\n";

        String xml = WSDLHelper.replaceSchemaLocations(testXML, "http://localhost:8080/test-service/wsdl/dgws/");
        assertTrue(xml.contains("http://localhost:8080/test-service/schema/wss/2004/01/testdir/test.xsd"));
    }

    @Test
    public void testReplaceLocationWithSingleQuote() {
        String testXML = "<xs:schema ...\n" +
                "            <xs:include schemaLocation='../schema/wss/2004/01/testdir/test.xsd'/>\n" +
                "         </xs:schema>\n";

        String xml = WSDLHelper.replaceSchemaLocations(testXML, "http://localhost:8080/test-service/wsdl/dgws");
        assertTrue(xml.contains("http://localhost:8080/test-service/schema/wss/2004/01/testdir/test.xsd"));
    }

    @Test
    public void testReplaceMultipleBackLocation() {
        String testXML = "<xs:schema ...\n" +
                "            <xs:include schemaLocation='../../../wss/2004/01/testdir/test.xsd'/>\n" +
                "         </xs:schema>\n";

        String xml = WSDLHelper.replaceSchemaLocations(testXML, "http://localhost:8080/test-service/schema/dir1/dir2/dir3/dir4/file.xsd");
        assertTrue(xml.contains("http://localhost:8080/test-service/schema/dir1/wss/2004/01/testdir/test.xsd"));
    }

    @Test
    public void testReplaceWithoutBackLocation() {
        String testXML = "<xs:schema ...\n" +
                "            <xs:include schemaLocation='testdir/test.xsd'/>\n" +
                "         </xs:schema>\n";

        String xml = WSDLHelper.replaceSchemaLocations(testXML, "http://localhost:8080/test-service/schema/test/file.xsd");
        assertTrue(xml.contains("http://localhost:8080/test-service/schema/test/testdir/test.xsd"));
    }
}
