/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.webservice;

import dk.sundhedsdatastyrelsen.livstestamente.domain.RequestContextVO;
import dk.sundhedsdatastyrelsen.livstestamente.domain.UserType;
import dk.sundhedsdatastyrelsen.livstestamente.service.util.TestHelper;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sundhedsdatastyrelsen.livstestamente.service.AbstractSpringTest;
import dk.sundhedsdatastyrelsen.livstestamente.service.util.TestHelperV2;
import dk.sundhedsdatastyrelsen.livstestamente.context.ServiceActor;

public abstract class AbstractWebServiceTest extends AbstractSpringTest {

	@Autowired
	TestHelper testHelper;

	@Autowired
	TestHelper.HL7Helper hl7Helper;

	@Autowired
    TestHelperV2 testHelperV2;

    @Autowired
    TestHelperV2.HL7Helper hl7HelperV2;



    RequestContextVO getSecurityDataForDataOwner(String personIdentifier) {
    	
    	ServiceActor serviceActor = new ServiceActor(UserType.Citizen);
    	serviceActor.setPersonIdentifier(personIdentifier);
		serviceActor.setUserTitle("borger");
		RequestContextVO requestContext = new RequestContextVO(serviceActor, Mockito.mock(SLALogItem.class));
		requestContext.setPersonIdentifier(personIdentifier);
		return requestContext;
    }

    protected RequestContextVO getSecurityDataForAdminUser(String citizenId, String personIdentifier) {

    	ServiceActor serviceActor = new ServiceActor(UserType.Admin);
    	serviceActor.setPersonIdentifier(personIdentifier);
		RequestContextVO requestContext = new RequestContextVO(serviceActor, Mockito.mock(SLALogItem.class));
		requestContext.setPersonIdentifier(citizenId);
		return requestContext;

    }

    protected RequestContextVO getSecurityDataForSystemUser() {

    	ServiceActor serviceActor = new ServiceActor(UserType.SystemUser);
		RequestContextVO requestContext = new RequestContextVO(serviceActor, Mockito.mock(SLALogItem.class));
		return requestContext;
    }
}
