/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.webservice;

import dk.sundhedsdatastyrelsen.livstestamente.service.AlarmServlet;
import dk.sundhedsdatastyrelsen.livstestamente.util.HealthIndicator;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashSet;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

@RunWith(MockitoJUnitRunner.class)
public class AlarmServletTest {

    @Mock
    HealthIndicator healthIndicator;

    @Spy
    Collection<HealthIndicator> healthIndicators = new HashSet<>();

    @InjectMocks
    AlarmServlet alarmServlet;

    MockHttpServletRequest request;
    MockHttpServletResponse response;

    @Before
    public void setUp() throws SQLException {
        request = new MockHttpServletRequest();
        response = new MockHttpServletResponse();

        healthIndicators.add(healthIndicator);
    }

    @Test
    public void isAlive_code200() throws IOException {
        Mockito.when(healthIndicator.getStatusCode()).thenReturn(200);

        alarmServlet.doGet(request, response);

        assertEquals(200, response.getStatus());
        assertNull(response.getContentType());
        assertEquals("", response.getContentAsString());
    }

    @Test
    public void isAlive_code500() throws IOException {
        Mockito.when(healthIndicator.getStatusCode()).thenReturn(500);
        Mockito.when(healthIndicator.getName()).thenReturn("isAlive_code500");
        Mockito.when(healthIndicator.getStatusText()).thenReturn("Failed as expected");

        alarmServlet.doGet(request, response);

        System.out.println(response.getContentAsString());

        assertEquals(500, response.getStatus());
        assertEquals("text/plain", response.getContentType());
        assertEquals("500  Non-Authoritative Information from: isAlive_code500\nFailed as expected\n", response.getContentAsString());
    }
}
