/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.webservice;

import dk.sundhedsdatastyrelsen.livstestamente.service.AbstractSpringTest;
import dk.sundhedsdatastyrelsen.livstestamente.service.DKSServlet;
import dk.sundhedsdatastyrelsen.livstestamente.service.http.MockLtrBtrHttpServletRequest;
import dk.sundhedsdatastyrelsen.livstestamente.service.http.MockLtrBtrHttpServletResponse;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import javax.servlet.ServletException;
import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class DKSServletWebServiceTests extends AbstractSpringTest {

    @Autowired
    private DKSServlet serviceUnderTest;

    @Test
    public void dksConfigLTR_code200() throws IOException, ServletException {
        // Arrange
        MockHttpServletRequest request = new MockLtrBtrHttpServletRequest();
        MockHttpServletResponse response = new MockLtrBtrHttpServletResponse();
        request.setServletPath("/dksconfig/ltr");

        // Act
        serviceUnderTest.doGet(request, response);

        // Assert
        assertEquals(200, response.getStatus());
        assertTrue(response.getContentAsString().contains("<name>LTR</name>"));
    }

    @Test
    public void dksConfigBTR_code200() throws IOException, ServletException {
        // Arrange
        MockHttpServletRequest request = new MockLtrBtrHttpServletRequest();
        MockHttpServletResponse response = new MockLtrBtrHttpServletResponse();
        request.setServletPath("/dksconfig/btr");

        // Act
        serviceUnderTest.doGet(request, response);

        // Assert
        assertEquals(200, response.getStatus());
        assertTrue(response.getContentAsString().contains("<name>BTR</name>"));
    }

    @Test
    public void dksConfigBTRv2_code200() throws IOException, ServletException {
        // Arrange
        MockHttpServletRequest request = new MockLtrBtrHttpServletRequest();
        MockHttpServletResponse response = new MockLtrBtrHttpServletResponse();
        request.setServletPath("/dksconfig/btrV2");

        // Act
        serviceUnderTest.doGet(request, response);

        // Assert
        assertEquals(200, response.getStatus());
        assertTrue(response.getContentAsString().contains("<name>BTR</name>"));
    }
}
