/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.service.webservice;

import dk.sundhedsdatastyrelsen.livstestamente.service.IsAliveServlet;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import javax.sql.DataSource;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import static org.junit.Assert.assertEquals;

@RunWith(MockitoJUnitRunner.class)
public class IsAliveServletTest {
    @Mock
    DataSource btrDataSource;
    @Mock
    Connection connection;
    @Mock
    Statement statement;

    @InjectMocks
    IsAliveServlet isAliveServlet;

    MockHttpServletRequest request;
    MockHttpServletResponse response;

    @Before
    public void setUp() throws SQLException {
        Mockito.when(btrDataSource.getConnection()).thenReturn(connection);
        Mockito.when(connection.createStatement()).thenReturn(statement);

        request = new MockHttpServletRequest();
        response = new MockHttpServletResponse();
    }

    @Test
    public void isAlive_code200() throws IOException, SQLException {
        Mockito.doReturn(true).when(statement).execute("SELECT 1");

        isAliveServlet.doGet(request, response);

        Mockito.verify(statement, Mockito.times(1)).execute("SELECT 1");

        assertEquals(200, response.getStatus());
        assertEquals("text/plain", response.getContentType());
        assertEquals("200  OK", response.getContentAsString().substring(0,7));
    }

    @Test
    public void isAlive_code500() throws IOException, SQLException {
        Mockito.doThrow(new SQLException()).when(statement).execute("SELECT 1");

        isAliveServlet.doGet(request, response);

        Mockito.verify(statement, Mockito.times(1)).execute("SELECT 1");

        assertEquals(500, response.getStatus());
        assertEquals("text/plain", response.getContentType());
        assertEquals("500 Internal Server Error: Database check failed", response.getContentAsString().substring(0,48));
    }
}
