/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.datacreator;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.time.Instant;

@Component
@PropertySource("classpath:application.properties")
public class DataCreatorProperties {

    @Value("${data-creator.person-file-name:}")
    private String personFileName;

    @Value("${data-creator.ltr.data-time-interval-start:}")
    private String ltrDataTimeIntervalStart;

    @Value("${data-creator.ltr.data-time-interval-end:}")
    private String ltrDataTimeIntervalEnd;

    @Value("${data-creator.ltr.chance-of-updates:}")
    private int ltrChanceOfUpdates;

    @Value("${data-creator.btr.data-time-interval-start:}")
    private String btrDataTimeIntervalStart;

    @Value("${data-creator.btr.data-time-interval-end:}")
    private String btrDataTimeIntervalEnd;

    @Value("${data-creator.btr.chance-of-updates:}")
    private int btrChanceOfUpdates;

    private final DataCreationProperties ltr = new DataCreationProperties();
    private final DataCreationProperties btr = new DataCreationProperties();

    @PostConstruct
    private void init() {
        ltr.setDataTimeIntervalStart(Instant.parse(ltrDataTimeIntervalStart));
        ltr.setDataTimeIntervalEnd(Instant.parse(ltrDataTimeIntervalEnd));
        ltr.setChanceOfUpdates(ltrChanceOfUpdates);
        btr.setDataTimeIntervalStart(Instant.parse(btrDataTimeIntervalStart));
        btr.setDataTimeIntervalEnd(Instant.parse(btrDataTimeIntervalEnd));
        btr.setChanceOfUpdates(btrChanceOfUpdates);
    }

    public String getPersonFileName() {
        return personFileName;
    }

    public DataCreationProperties getLtr() {
        return ltr;
    }

    public DataCreationProperties getBtr() {
        return btr;
    }

    @Override
    public String toString() {
        return "DataCreatorProperties{" +
                "personFileName='" + personFileName + '\'' +
                ", ltr=" + ltr +
                ", btr=" + btr +
                '}';
    }

    public static class DataCreationProperties {
        private Instant dataTimeIntervalStart;
        private Instant dataTimeIntervalEnd;

        private int chanceOfUpdates;

        public Instant getDataTimeIntervalStart() {
            return dataTimeIntervalStart;
        }

        public void setDataTimeIntervalStart(Instant dataTimeIntervalStart) {
            this.dataTimeIntervalStart = dataTimeIntervalStart;
        }

        public Instant getDataTimeIntervalEnd() {
            return dataTimeIntervalEnd;
        }

        public void setDataTimeIntervalEnd(Instant dataTimeIntervalEnd) {
            this.dataTimeIntervalEnd = dataTimeIntervalEnd;
        }

        public int getChanceOfUpdates() {
            return chanceOfUpdates;
        }

        public void setChanceOfUpdates(int chanceOfUpdates) {
            this.chanceOfUpdates = chanceOfUpdates;
        }

        @Override
        public String toString() {
            return "DataCreationProperties{" +
                    "dataTimeIntervalStart=" + dataTimeIntervalStart +
                    ", dataTimeIntervalEnd=" + dataTimeIntervalEnd +
                    ", chanceOfUpdates=" + chanceOfUpdates +
                    '}';
        }
    }
}
