/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.datacreator;

import dk.sundhedsdatastyrelsen.livstestamente.datacreator.business.LivingWillDataCreator;
import dk.sundhedsdatastyrelsen.livstestamente.datacreator.business.TreatmentWillDataCreator;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

@Component
@Profile("!test")
public class Runner {

    private static final Logger log = LogManager.getLogger(Runner.class);

    private LivingWillDataCreator livingWillDataCreator;
    private TreatmentWillDataCreator treatmentWillDataCreator;
    private DataCreatorProperties dataCreatorProperties;

    public Runner(LivingWillDataCreator livingWillDataCreator, TreatmentWillDataCreator treatmentWillDataCreator, DataCreatorProperties dataCreatorProperties) {
        this.livingWillDataCreator = livingWillDataCreator;
        this.treatmentWillDataCreator = treatmentWillDataCreator;
        this.dataCreatorProperties = dataCreatorProperties;
    }

    @PostConstruct
    public void run() throws Exception {
        log.info("Data creator says Hello World, and begins!");

        log.info(dataCreatorProperties);

        livingWillDataCreator.start();
        treatmentWillDataCreator.start();

        log.info("Data creator says goodbye!");
    }
}
