/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.datacreator.util;

import dk.sundhedsdatastyrelsen.livstestamente.domain.LivingWillVO;

import java.time.Instant;
import java.time.Year;
import java.time.ZoneOffset;

public class LivingWillDataUtil {

    private static final Instant validTo = Year.of(8099).atDay(1).atStartOfDay(ZoneOffset.UTC).toInstant();

    public static LivingWillVO getNewLivingWill(String personIdentifier, Instant timeOfFirstData, Instant timeOfLastData) {
        final Instant currentTime = RandomDataUtil.getRandomInstantInInterval(timeOfFirstData, timeOfLastData);

        final LivingWillVO livingWill = new LivingWillVO();

        // from param
        livingWill.setPersonIdentifier(personIdentifier);

        setRandomValues(livingWill);

        // fixed
        livingWill.setVersion(1);

        // random but same for both fields
        livingWill.setModifiedDateTime(currentTime);
        livingWill.setValidFrom(currentTime);

        // fixed
        livingWill.setValidTo(validTo);

        return livingWill;
    }

    public static LivingWillVO getUpdatedLivingWillData(LivingWillVO currentLivingWill) {
        final Instant oldModifiedDateTime = currentLivingWill.getModifiedDateTime();
        final Instant newModifiedDateTime =
                RandomDataUtil.getRandomInstantInInterval(oldModifiedDateTime, oldModifiedDateTime.plusSeconds(3600));

        final LivingWillVO livingWill = new LivingWillVO();

        // keep
        livingWill.setPersonIdentifier(currentLivingWill.getPersonIdentifier());

        setRandomValues(livingWill);

        // bump
        livingWill.setVersion(currentLivingWill.getVersion() + 1);

        // random but same for both fields
        livingWill.setModifiedDateTime(newModifiedDateTime);
        livingWill.setValidFrom(newModifiedDateTime);

        // fixed
        livingWill.setValidTo(validTo);

        return livingWill;
    }

    private static void setRandomValues(LivingWillVO livingWill) {
        // random
        livingWill.setNoLifeProlongingIfDying(RandomDataUtil.getRandomBoolean());

        // random
        livingWill.setNoLifeProlongingIfSeverelyDegraded(RandomDataUtil.getRandomBoolean());
    }
}
