/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.datacreator.util;

import java.time.Instant;
import java.util.Random;

public final class RandomDataUtil {

    private static final Random random = new Random();

    static Instant getRandomInstantInInterval(Instant startOfInterval, Instant endOfInterval) {
        if (startOfInterval.isAfter(endOfInterval)) {
            throw new IllegalArgumentException("startOfInterval must be before endOfInterval");
        }

        final long intervalInSeconds = endOfInterval.getEpochSecond() - startOfInterval.getEpochSecond();
        final int randomNumberOfSeconds = getRandomNumber(0, Math.toIntExact(intervalInSeconds));

        return startOfInterval.plusSeconds(randomNumberOfSeconds);
    }

    static int getRandomNumber(int min, int max) {
        if (min >= max) {
            throw new IllegalArgumentException("max must be greater than min");
        }

        return random.nextInt((max - min) + 1) + min;
    }

    static boolean getRandomBoolean() {
        return random.nextBoolean();
    }

    public static boolean getRandomDecision(int trueChance) {
        return RandomDataUtil.getRandomNumber(1, 100) <= trueChance;
    }
}
