/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.livstestamente.datacreator.util;

import dk.sundhedsdatastyrelsen.livstestamente.domain.AcceptanceNeeded;
import dk.sundhedsdatastyrelsen.livstestamente.domain.TreatmentWillVO;

import java.time.Instant;
import java.time.Year;
import java.time.ZoneOffset;

public class TreatmentWillDataUtil {

    private static final Instant validTo = Year.of(8099).atDay(1).atStartOfDay(ZoneOffset.UTC).toInstant();

    public static TreatmentWillVO getNewTreatmentWill(String personIdentifier, Instant timeOfFirstData, Instant timeOfLastData) {
        final Instant currentTime = RandomDataUtil.getRandomInstantInInterval(timeOfFirstData, timeOfLastData);

        TreatmentWillVO treatmentWill = new TreatmentWillVO();

        // from param
        treatmentWill.setPersonIdentifier(personIdentifier);

        setRandomValues(treatmentWill);

        // fixed
        treatmentWill.setVersion(1);

        // random but same for both fields
        treatmentWill.setModifiedDateTime(currentTime);
        treatmentWill.setValidFrom(currentTime);

        // fixed
        treatmentWill.setValidTo(validTo);

        return treatmentWill;
    }

    public static TreatmentWillVO getUpdatedTreatmentWillData(TreatmentWillVO currentTreatmentWill) {
        final Instant oldModifiedDateTime = currentTreatmentWill.getModifiedDateTime();
        final Instant newModifiedDateTime =
                RandomDataUtil.getRandomInstantInInterval(oldModifiedDateTime, oldModifiedDateTime.plusSeconds(3600));

        final TreatmentWillVO treatmentWill = new TreatmentWillVO();

        // keep
        treatmentWill.setPersonIdentifier(currentTreatmentWill.getPersonIdentifier());

        setRandomValues(treatmentWill);

        // bump
        treatmentWill.setVersion(currentTreatmentWill.getVersion() + 1);

        // random but same for both fields
        treatmentWill.setModifiedDateTime(newModifiedDateTime);
        treatmentWill.setValidFrom(newModifiedDateTime);

        // fixed
        treatmentWill.setValidTo(validTo);

        return treatmentWill;
    }

    private static void setRandomValues(TreatmentWillVO treatmentWill) {
        // random
        final boolean randomBoolean1 = RandomDataUtil.getRandomBoolean();
        treatmentWill.setNoLifeProlongingIfDying(randomBoolean1);

        // random
        final boolean randomBoolean2 = RandomDataUtil.getRandomBoolean();
        treatmentWill.setNoLifeProlongingIfSeverelyDegraded(randomBoolean2);
        if (randomBoolean2) {
            treatmentWill.setNoLifeProlongingIfSeverelyDegradedAcceptanceNeeded(getRandomAcceptanceNeeded());
        }

        // random
        final boolean randomBoolean3 = RandomDataUtil.getRandomBoolean();
        treatmentWill.setNoLifeProlongingIfSeverePain(randomBoolean3);
        if (randomBoolean3) {
            treatmentWill.setNoLifeProlongingIfSeverePainAcceptanceNeeded(getRandomAcceptanceNeeded());
        }

        // random
        final boolean randomBoolean4 = RandomDataUtil.getRandomBoolean();
        treatmentWill.setNoForcedTreatmentIfIncapable(randomBoolean4);
        if (randomBoolean4) {
            treatmentWill.setNoForcedTreatmentIfIncapableAcceptanceNeeded(getRandomAcceptanceNeeded());
        }
    }

    private static AcceptanceNeeded getRandomAcceptanceNeeded() {
        final AcceptanceNeeded[] values = AcceptanceNeeded.values();
        final int numberOfValues = values.length;
        final int randomNumber = RandomDataUtil.getRandomNumber(0, numberOfValues);

        if (randomNumber == numberOfValues) {
            // Should also be able to return null
            return null;
        } else {
            return values[randomNumber];
        }
    }
}
